# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  LSE
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
  Description:  This file is part of the privacyidea service
                HOTP token
  Dependencies: -

'''
import binascii
import logging
import time
import datetime

import traceback

from privacyidea.lib.config  import getFromConfig

from privacyidea.lib.crypto   import decryptPin, encryptPin
from privacyidea.lib.crypto   import kdf2
from privacyidea.lib.crypto   import createNonce

from privacyidea.lib.policy  import PolicyClass

### TODO: move this as ocra specific methods
from privacyidea.lib.token import getRolloutToken4User
from privacyidea.lib.util import normalize_activation_code

from privacyidea.lib.ocra    import OcraSuite

from privacyidea.lib.validate import create_challenge
from privacyidea.lib.validate import get_challenges
from privacyidea.lib.reply   import create_img

from pylons.i18n.translation import _
from pylons import request, config, tmpl_context as c
from privacyidea.lib.config import get_privacyIDEA_config
from privacyidea.lib.tokenclass import TokenClass
from privacyidea.lib.log import log_with

# needed for ocra token
import urllib


optional = True
required = False

log = logging.getLogger(__name__)
#### Ocra2TokenClass #####################################


class Ocra2TokenClass(TokenClass):
    '''
    Ocra2TokenClass  implement an ocra compliant token

    used from Config
        OcraMaxChallenges         - number of open challenges per token
                                            if None: 3
        Ocra2ChallengeValidityTime  timeout definition in seconds
        OcraDefaultSuite          - if none :'OCRA-1:HOTP-SHA256-8:C-QN08'
        QrOcraDefaultSuite        - if none :'OCRA-1:HOTP-SHA256-8:C-QA64'


    algorithm Ocra Token Rollout: tow phases of rollout

    1. https://privacyideaserver/admin/init?
        type=ocra&
        genkey=1&
        sharedsecret=1&
        user=BENUTZERNAME&
        session=SESSIONKEY

        =>> "serial" : SERIENNUMMER, "sharedsecret" : DATAOBJECT, "app_import" : IMPORTURL
        - genSharedSecret - vom HSM oder urandom ?
        - app_import : + privacyidea://
                       + ocrasuite ->> default aus dem config: (DefaultOcraSuite)
                       + sharedsecret (Länge wie ???)
                       + seriennummer
        - seriennummer: uuid
        - token wird angelegt ist aber nicht aktiv!!! (counter == 0)


    2. https://privacyideaserver/admin/init?
        type=ocra&
        genkey=1&
        activationcode=AKTIVIERUNGSCODE&
        user=BENUTZERNAME&
        message=MESSAGE&
        session=SESSIONKEY

        =>> "serial" : SERIENNUMMER, "nonce" : DATAOBJECT, "transactionid" : "TRANSAKTIONSID, "app_import" : IMPORTURL

        - nonce - von HSM oder random ?
        - pkcs5 - kdf2
        - es darf zur einer Zeit nur eine QR Token inaktiv (== im Ausrollzustand) sein !!!!!
          der Token wird über den User gefunden
        - seed = pdkdf2(nonce + activcode + shared secret)
        - challenge generiern - von urandom oder HSM

    3. check_t
        - counter ist > nach der ersten Transaktion
        - if counter >= 1: delete sharedsecret löschen


    '''

    @classmethod
    @log_with(log)
    def getClassType(cls):
        '''
        getClassType - return the token type shortname

        :return: 'ocra2'
        :rtype: string
        '''
        return "ocra2"

    @classmethod
    def getClassPrefix(cls):
        return "LSO2"

    @classmethod
    def classInit(cls, param, user=None):

        helper_param = {}

        tok_type = "ocra2"

        ## take the keysize from the ocrasuite
        ocrasuite = param.get("ocrasuite", None)
        activationcode = param.get("activationcode", None)
        sharedsecret = param.get("sharedsecret", None)
        serial = param.get("serial", None)
        genkey = param.get("genkey", None)

        if activationcode is not None:
            ## dont create a new key
            genkey = None
            serial = getRolloutToken4User(user=user, serial=serial, tok_type=tok_type)
            if serial is None:
                raise Exception('no token found for user: %r or serial: %r' % (user, serial))
            helper_param['serial'] = serial
            helper_param['activationcode'] = normalize_activation_code(activationcode)

        if ocrasuite is None:
            if sharedsecret is not None or  activationcode is not None:
                ocrasuite = getFromConfig("QrOcraDefaultSuite", 'OCRA-1:HOTP-SHA256-6:C-QA64')
            else:
                ocrasuite = getFromConfig("OcraDefaultSuite", 'OCRA-1:HOTP-SHA256-8:C-QN08')
            helper_param['ocrasuite'] = ocrasuite

        if genkey is not None:
            if ocrasuite.find('-SHA256'):
                key_size = 32
            elif ocrasuite.find('-SHA512'):
                key_size = 64
            else:
                key_size = 20
            helper_param['key_size'] = key_size

        return helper_param





    @classmethod
    def getClassInfo(cls, key=None, ret='all'):
        '''
        getClassInfo - returns all or a subtree of the token definition

        :param key: subsection identifier
        :type key: string

        :param ret: default return value, if nothing is found
        :type ret: user defined

        :return: subsection if key exists or user defined
        :rtype : s.o.

        '''

        res = {
               'type' : 'ocra2',
               'title' : _('OCRA2 Token'),
               'description' :
                    _('ocra challenge-response token - hmac event based'),
               'init'         : { 'title'  : {'html'      : 'ocra2token.mako',
                                             'scope'     : 'enroll.title', },
                                  'page' : {'html'      : 'ocra2token.mako',
                                            'scope'      : 'enroll', },
                                   },

               'config'         : {'title'  : {'html'      : 'ocra2token.mako',
                                             'scope'     : 'config.title', },
                                   'page' : {'html'      : 'ocra2token.mako',
                                            'scope'      : 'config', },
                                   },

               'selfservice'   :  { 'enroll' :
                                   {'title'  :
                                    { 'html'      : 'ocra2token.mako',
                                      'scope'     : 'selfservice.title.enroll',
                                      },
                                    'page' :
                                    {'html'       : 'ocra2token.mako',
                                     'scope'      : 'selfservice.enroll',
                                     },
                                    },
                                   'activate_OCRA2' :
                                   {'title'  :
                                    { 'html'      : 'ocra2token.mako',
                                      'scope'     : 'selfservice.title.activate',
                                      },
                                    'page' :
                                    {'html'       : 'ocra2token.mako',
                                     'scope'      : 'selfservice.activate',
                                     },
                                    },

                                  },

            'policy' : { 'selfservice' : {'activate_OCRA2' : {'type' : 'bool'} }}

        }

        if key is not None and res.has_key(key):
            ret = res.get(key)
        else:
            if ret == 'all':
                ret = res

        return ret


    @log_with(log)
    def __init__(self, aToken):
        '''
        getInfo - return the status of the token rollout

        :return: info of the ocra token state
        :rtype: dict
        '''
        TokenClass.__init__(self, aToken)
        self.setType(u"ocra2")
        self.transId = 0
        self.Policy = PolicyClass(request, config, c,
                                  get_privacyIDEA_config())

        self.mode = ['challenge']
        return

    @log_with(log)
    def getInfo(self):
        '''
        getInfo - return the status of the token rollout

        :return: info of the ocra token state
        :rtype: dict
        '''
        return self.info

    @log_with(log)
    def update(self, params, reset_failcount=True):
        '''
        update: add further definition for token from param in case of init
        '''
        if params.has_key('ocrasuite'):
            self.ocraSuite = params.get('ocrasuite')
        else:
            activationcode = params.get('activationcode', None)
            sharedSecret = params.get('sharedsecret', None)


            if activationcode is None and sharedSecret is None:
                self.ocraSuite = self.getOcraSuiteSuite()
            else:
                self.ocraSuite = self.getQROcraSuiteSuite()

        if params.get('activationcode', None):
            ## due to changes in the tokenclass parameter handling
            ## we have to add for compatibility a genkey parameter
            if params.has_key('otpkey') == False and params.has_key('genkey') == False:
                log.warning('missing parameter genkey\
                             to complete the rollout 2!')
                params['genkey'] = 1


        TokenClass.update(self, params, reset_failcount=reset_failcount)

        self.addToTokenInfo('ocrasuite', self.ocraSuite)

        ocraSuite = OcraSuite(self.ocraSuite)
        otplen = ocraSuite.truncation
        self.setOtpLen(otplen)

        ocraPin = params.get('ocrapin', None)
        if ocraPin is not None:
            self.token.setUserPin(ocraPin)

        if params.has_key('otpkey'):
            self.setOtpKey(params.get('otpkey'))

        self._rollout_1(params)
        self._rollout_2(params)

        return


    @log_with(log)
    def _rollout_1(self, params):
        '''
        do the rollout 1 step

        1. https://privacyideaserver/admin/init?
            type=ocra&
            genkey=1&
            sharedsecret=1&
            user=BENUTZERNAME&
            session=SESSIONKEY

            =>> "serial" : SERIENNUMMER, "sharedsecret" : DATAOBJECT, "app_import" : IMPORTURL
            - genSharedSecret - vom HSM oder urandom ?
            - app_import : + privacyidea://
                           + ocrasuite ->> default aus dem config: (DefaultOcraSuite)
                           + sharedsecret (Länge wie ???)
                           + seriennummer
            - seriennummer: uuid ??
            - token wird angelegt ist aber nicht aktiv!!! (counter == 0)

        '''
        sharedSecret = params.get('sharedsecret', None)
        if sharedSecret == '1':
            ##  preserve the rollout state
            self.addToTokenInfo('rollout', '1')

            ##  preseerver the current key as sharedSecret
            secObj = self.token.getHOtpKey()
            key = secObj.getKey()
            encSharedSecret = encryptPin(key)
            self.addToTokenInfo('sharedSecret', encSharedSecret)

            info = {}
            uInfo = {}

            info['sharedsecret'] = key
            uInfo['sh'] = key

            info['ocrasuite'] = self.getOcraSuiteSuite()
            uInfo['os'] = self.getOcraSuiteSuite()

            info['serial'] = self.getSerial()
            uInfo['se'] = self.getSerial()

            info['app_import'] = 'lseqr://init?%s' % (urllib.urlencode(uInfo))
            del info['ocrasuite']
            self.info = info

            self.token.privacyIDEAIsactive = False

        return

    @log_with(log)
    def _rollout_2(self, params):
        '''
        2.

        https://privacyideaserver/admin/init?
            type=ocra&
            genkey=1&
            activationcode=AKTIVIERUNGSCODE&
            user=BENUTZERNAME&
            message=MESSAGE&
            session=SESSIONKEY

        =>> "serial" : SERIENNUMMER, "nonce" : DATAOBJECT, "transactionid" : "TRANSAKTIONSID, "app_import" : IMPORTURL

        - nonce - von HSM oder random ?
        - pkcs5 - kdf2
        - es darf zur einer Zeit nur eine QR Token inaktiv (== im Ausrollzustand) sein !!!!!
          der Token wird über den User gefunden
        - seed = pdkdf2(nonce + activcode + shared secret)
        - challenge generiern - von urandom oder HSM

        '''
        activationcode = params.get('activationcode', None)
        if activationcode is not None:

            ##  genkey might have created a new key, so we have to rely on
            encSharedSecret = self.getFromTokenInfo('sharedSecret', None)
            if encSharedSecret is None:
                raise Exception ('missing shared secret of initialition for token %r' % (self.getSerial()))

            sharedSecret = decryptPin(encSharedSecret)

            ##  we generate a nonce, which in the end is a challenge
            nonce = createNonce()
            self.addToTokenInfo('nonce', nonce)

            ##  create a new key from the ocrasuite
            key_len = 20
            if self.ocraSuite.find('-SHA256'):
                key_len = 32
            elif self.ocraSuite.find('-SHA512'):
                key_len = 64


            newkey = kdf2(sharedSecret, nonce, activationcode, key_len)
            self.setOtpKey(binascii.hexlify(newkey))

            ##  generate challenge, which is part of the app_import
            message = params.get('message', None)

            #(transid, challenge, _ret, url) = self.challenge(message)

            #self.createChallenge()
            (res, opt) = create_challenge(self, options=params)

            challenge = opt.get('challenge')
            url = opt.get('url')
            transid = opt.get('transactionid')

            ##  generate response
            info = {}
            uInfo = {}
            info['serial'] = self.getSerial()
            uInfo['se'] = self.getSerial()
            info['nonce'] = nonce
            uInfo['no'] = nonce
            info['transactionid'] = transid
            uInfo['tr'] = transid
            info['challenge'] = challenge
            uInfo['ch'] = challenge
            if message is not None:
                uInfo['me'] = str(message.encode("utf-8"))

            ustr = urllib.urlencode({'u':str(url.encode("utf-8"))})
            uInfo['u'] = ustr[2:]
            info['url'] = str(url.encode("utf-8"))

            app_import = 'lseqr://nonce?%s' % (urllib.urlencode(uInfo))

            ##  add a signature of the url
            signature = {'si': self.signData(app_import) }
            info['signature'] = signature.get('si')

            info['app_import'] = "%s&%s" % (app_import, urllib.urlencode(signature))
            self.info = info

            ##  setup new state
            self.addToTokenInfo('rollout', '2')
            self.enable(True)

        return

    @log_with(log)
    def getOcraSuiteSuite(self):
        '''
        getQROcraSuiteSuite - return the QR Ocra Suite - if none, it will return the default

        :return: Ocrasuite of token
        :rtype: string
        '''

        defaultOcraSuite = getFromConfig("OcraDefaultSuite", 'OCRA-1:HOTP-SHA256-8:C-QN08')
        self.ocraSuite = self.getFromTokenInfo('ocrasuite', defaultOcraSuite)

        return self.ocraSuite

    @log_with(log)
    def getQROcraSuiteSuite(self):
        '''
        getQROcraSuiteSuite - return the QR Ocra Suite - if none, it will return the default

        :return: QROcrasuite of token
        :rtype: string
        '''
        defaultOcraSuite = getFromConfig("QrOcraDefaultSuite", 'OCRA-1:HOTP-SHA256-8:C-QA64')
        self.ocraSuite = self.getFromTokenInfo('ocrasuite', defaultOcraSuite)

        return self.ocraSuite


    @log_with(log)
    def signData(self, data):
        '''
        sign the received data with the secret key

        :param data: arbitrary string object
        :type param: string

        :return: hexlified signature of the data
        '''
        secretHOtp = self.token.getHOtpKey()
        ocraSuite = OcraSuite(self.getOcraSuiteSuite(), secretHOtp)
        signature = ocraSuite.signData(data)
        return signature

    @log_with(log)
    def verify_challenge_is_valid(self, challenge, session):
        '''
        verify, if a challenge is valid according to the ocrasuite definition
        of the token
        '''

        ret = True

        counter = self.getOtpCount()

        secretHOtp = self.token.getHOtpKey()
        ocraSuite = OcraSuite(self.getOcraSuiteSuite(), secretHOtp)

        ## set the pin onyl in the compliant hashed mode
        pin = ''
        if ocraSuite.P is not None:
            pinObj = self.token.getUserPin()
            pin = pinObj.getKey()

        try:
            param = {}
            param['C'] = counter
            param['Q'] = challenge
            param['P'] = pin
            param['S'] = session
            if ocraSuite.T is not None:
                now = datetime.datetime.now()
                stime = now.strftime("%s")
                itime = int(stime)
                param['T'] = itime

            ''' verify that the data is compliant with the OcraSuitesuite
                and the client is able to calc the otp
            '''
            c_data = ocraSuite.combineData(**param)
            ocraSuite.compute(c_data)

        except Exception as ex:
            log.error("challenge verification failed: "
                                "%s,%r: " % (challenge, ex))
            log.error(traceback.format_exc())
            ret = False

        return ret

    @log_with(log)
    def createChallenge(self, state, options=None):
        '''
        standard API to create an ocra challenge
        '''
        res = True

        ## which kind of challenge gen should be used
        typ = 'raw'

        input = None
        challenge = None
        session = None
        message = ""

        if options is not None:
            input = options.get('challenge', None)
            if input is None:
                input = options.get('message', None)
            if input is None:
                input = options.get('data', None)

            typ = options.get('challenge_type', 'raw')
            ## ocra token could contain a session attribute
            session = options.get('ocra_session', None)

        if input is None or len(input) == 0:
            typ = 'random'

        secretHOtp = self.token.getHOtpKey()
        ocraSuite = OcraSuite(self.getOcraSuiteSuite(), secretHOtp)

        if typ == 'raw':
            challenge = ocraSuite.data2rawChallenge(input)
        elif typ == 'random':
            challenge = ocraSuite.data2randomChallenge(input)
        elif typ == 'hash':
            challenge = ocraSuite.data2hashChallenge(input)

        log.debug('challenge: %r ' % (challenge))

        store_data = {
                'challenge' : "%s" % (challenge),
                'serial' : self.token.getSerial(),
                'input' : '',
                'url' : '',
                }

        if input is not None:
            store_data['input'] = input

        if session is not None:
            store_data["session"] = session

        res = self.verify_challenge_is_valid(challenge, session)

        ## add Info: so depending on the Info, the rendering could be done
        ##          as a callback into the token via
        ##          token.getQRImageData(opt=details)
        realms = self.token.getRealms()
        if len(realms) > 0:
            store_data["url"] = self.Policy.get_qrtan_url(realms[0].name)

        ## we will return a dict of all
        attributes = self.prepare_message(store_data, state)
        attributes['challenge'] = challenge

        if attributes != None and "data" in attributes:
            message = attributes.get("data")
            del attributes['data']

        return (res, message, store_data, attributes)

    def prepare_message(self, data, transId):
        '''
        prepare the challenge response message

        :param data:
        :param transId: the transaction/state refenence id
        remark:
        we need the state/transId in the inner scope to support the signing
        of the whole request including the state/transId
        '''

        url = data.get("url")
        u = (str(urllib.urlencode({'u': '%s' % url})))
        u = urllib.urlencode({'u': "%s" % (url.encode("utf-8"))})

        challenge = data.get('challenge')
        input = data.get('input')

        uInfo = {'tr': transId,
                 'ch': challenge,
                 'me': str(input.encode("utf-8")),
                 'u': str(u[2:])}
        detail = {'request'         : str(input.encode("utf-8")),
                  'url'             : str(url.encode("utf-8")),
                 }

        ## create the app_url from the data
        dataobj = 'lseqr://req?%s' % (str(urllib.urlencode(uInfo)))

        ## append the signature to the url
        signature = {'si': self.signData(dataobj)}
        uInfo['si'] = signature
        dataobj = '%s&%s' % (dataobj, str(urllib.urlencode(signature)))

        detail["data"] = dataobj

        return detail

    @log_with(log)
    def challenge(self, data, session='', typ='raw', challenge=None):
        '''
        the challenge method is for creating an transaction / challenge object

        remark: the transaction has a maximum lifetime and a reference to
                the OcraSuite token (serial)

        :param data:     data, which is the base for the challenge or None
        :type data:     string or None
        :param session:  session support for ocratokens
        :type session:  string
        :type typ:      define, which kind of challenge base should be used
                         could be raw - take the data input as is
                               (extract chars accordind challenge definition Q)
                         or random    - will generate a random input
                         or hased     - will take the hash of the input data

        :return:    challenge response containing the transcation id and the
                    challenge for the ocrasuite
        :rtype :    tuple of (transId(string), challenge(string))


        '''
        secretHOtp = self.token.getHOtpKey()
        ocraSuite = OcraSuite(self.getOcraSuiteSuite(), secretHOtp)

        if data is None or len(data) == 0:
            typ = 'random'

        if challenge is None:
            if typ == 'raw':
                challenge = ocraSuite.data2rawChallenge(data)
            elif typ == 'random':
                challenge = ocraSuite.data2randomChallenge(data)
            elif typ == 'hash':
                challenge = ocraSuite.data2hashChallenge(data)

        log.debug('challenge: %r ' % (challenge))

        counter = self.getOtpCount()

        ## set the pin onyl in the compliant hashed mode
        pin = ''
        if ocraSuite.P is not None:
            pinObj = self.token.getUserPin()
            pin = pinObj.getKey()

        try:
            param = {}
            param['C'] = counter
            param['Q'] = challenge
            param['P'] = pin
            param['S'] = session
            if ocraSuite.T is not None:
                now = datetime.datetime.now()
                stime = now.strftime("%s")
                itime = int(stime)
                param['T'] = itime

            ''' verify that the data is compliant with the OcraSuitesuite
                and the client is able to calc the otp
            '''
            c_data = ocraSuite.combineData(**param)
            ocraSuite.compute(c_data)

        except Exception as ex:
            log.error(traceback.format_exc())
            raise Exception('[Ocra2TokenClass] Failed to create ocrasuite '
                                                        'challenge: %r' % (ex))

        ##  create a non exisiting challenge
        try:

            (res, opt) = create_challenge(self, options={'messgae': data})

            transid = opt.get('transactionid')
            challenge = opt.get('challenge')

        except Exception as ex:
            ##  this might happen if we have a db problem or
            ##   the uniqnes constrain does not fit
            log.error(traceback.format_exc())
            raise Exception('[Ocra2TokenClass] Failed to create '
                                                'challenge object: %s' % (ex))

        realm = None
        realms = self.token.getRealms()
        if len(realms) > 0:
            realm = realms[0]

        url = ''
        if realm is not None:
            url = self.Policy.get_qrtan_url(realm.name)

        return (transid, challenge, True, url)

### challenge interfaces starts here
    def is_challenge_request(self, passw, user, options=None):
        '''
        check, if the request would start a challenge

        - default: if the passw contains only the pin, this request would
        trigger a challenge

        - in this place as well the policy for a token is checked

        :param passw: password, which might be pin or pin+otp
        :param options: dictionary of additional request parameters

        :retrun: returns true or false
        '''

        request_is_valid = False

        if passw is None:
            ## for compatibility:
            # in case of ocra2, we accept to trigger a challenge even with an
            # missing password, if there is a challenge or data in the request
            if 'data' in options or 'challenge' in options:
                request_is_valid = True
        else:
            tok = super(Ocra2TokenClass, self)
            request_is_valid = tok.is_challenge_request(passw, user,
                                                        options=options)

        return request_is_valid

    def is_challenge_response(self, passw, user, options=None,
                                                            challenges=None):
        '''
        for the ocra token,

        :param passw: password, which might be pin or pin+otp
        :param user: the requesting user
        :param options: dictionary of additional request parameters

        :return: returns true or false
        '''

        challenge_response = False
        if (passw is not None and  len(passw) > 0 and
           challenges is not None and len(challenges) > 0):
            challenge_response = True

        if 'challenge' in options or 'data' in options:
            challenge_response = False

        ## we leave out the checkOtp, which is done later
        ## either in checkResponse4Challenge
        ## or in the check pin+otp

        return challenge_response

    def is_challenge_valid(self, challenge=None):
        '''
        this method proves the validity of a challenge
        - the default implementation tests, if the challegenge start
        is in the default vality time window.

        :param challenge: challenge object
        :return: true or false
        '''

        return True

    def checkResponse4Challenge(self, user, passw, options=None, challenges=None):
        '''
        verify the response of a previous challenge

        :param user:      the requesting user
        :param passw:     the to be checked pass: (otp) & trans_id | (pin+otp)
        :param options:   options an additional argument, which could be token
                          specific
        :param challenges: the list of challenges, where each challenge is
                            described as dict
        :return: tuple of (boolean and the list matching challenge ids)
        '''
        res = False
        otpcount = -1
        matching_challenges = []
        mids = {}
        loptions = {}

        if options is not None:
            loptions.update(options)
        if 'session' in loptions:
            del loptions['session']

        (res, pin, otpval) = self.splitPinPass(passw)
        res = self.checkPin(pin)

        if res == True:
            window = self.getCounterWindow()
            counter = self.getOtpCount()
            transids = set()

            ## preserve the provided transaction
            if 'transactionid' in options:
                transids.add(options.get('transactionid'))

            ## add all identified challenges by transid
            for challenge in challenges:
                ### checkOtp recieve the challenge in the options
                ### as transcationid
                try:
                    transid = challenge.get('transid', None)
                except Exception:
                    pass
                if transid is not None:
                    mids[transid] = challenge

            for transid in mids.keys():
                ## intentional overwrite the transaction which has been provided
                loptions['transactionid'] = transid
                otpcount = self.checkOtp(otpval, counter, window, options=loptions)
                if otpcount >= 0:
                    matching_challenges.append(mids.get(transid))
                    break

        return (otpcount, matching_challenges)

    @log_with(log)
    def checkOtp(self, passw , counter, window, options=None):
        '''
        checkOtp - standard callback of privacyidea to verify the token

        :param passw:      the passw / otp, which has to be checked
        :type passw:       string
        :param counter:    the start counter
        :type counter:     int
        :param  window:    the window, in which the token is valid
        :type  window:     int
        :param options:    options contains the transaction id,
                            eg. if check_t checks one transaction
                            this will support assynchreonous otp checks
                            (when check_t is used)
        :type options:     dict

        :return:           verification counter or -1
        :rtype:            int (-1)

        '''
        ret = -1

        challenges = []
        serial = self.getSerial()

        if options is None:
            options = {}

        maxRequests = int(getFromConfig("Ocra2MaxChallengeRequests", '3'))

        if 'transactionid' in options:
            transid = options.get('transactionid', None)
            challs = get_challenges(serial=serial, transid=transid)
            for chall in challs:
                (rec_tan, rec_valid) = chall.getTanStatus()
                if rec_tan == False:
                    challenges.append(chall)
                elif rec_valid == False:
                    ## add all touched but failed challenges
                    if chall.getTanCount() <= maxRequests:
                        challenges.append(chall)

        if 'challenge' in options:
            ## direct challenge - there might be addtionalget info like
            ## session data in the options
            challenges.append(options)

        if len(challenges) == 0:
            challs = get_challenges(serial=serial)
            for chall in challs:
                (rec_tan, rec_valid) = chall.getTanStatus()
                if rec_tan == False:
                    ## add all untouched challenges
                    challenges.append(chall)
                elif rec_valid == False:
                    ## add all touched but failed challenges
                    if chall.getTanCount() <= maxRequests:
                        challenges.append(chall)

        if len(challenges) == 0:
            err = 'No open transaction found for token %s' % serial
            log.error(err)  ##TODO should log and fail!!
            raise Exception(err)

        ## prepare the challenge check - do the ocra setup
        secretHOtp = self.token.getHOtpKey()
        ocraSuite = OcraSuite(self.getOcraSuiteSuite(), secretHOtp)

        ## set the ocra token pin
        ocraPin = ''
        if ocraSuite.P is not None:
            ocraPinObj = self.token.getUserPin()
            ocraPin = ocraPinObj.getKey()

            if ocraPin is None or len(ocraPin) == 0:
                ocraPin = ''

        timeShift = 0
        if  ocraSuite.T is not None:
            defTimeWindow = int(getFromConfig("ocra.timeWindow", 180))
            window = int(self.getFromTokenInfo('timeWindow', defTimeWindow)) / ocraSuite.T
            defTimeShift = int(getFromConfig("ocra.timeShift", 0))
            timeShift = int(self.getFromTokenInfo("timeShift", defTimeShift))

        default_retry_window = int(getFromConfig("ocra2.max_check_challenge_retry", 0))
        retry_window = int(self.getFromTokenInfo("max_check_challenge_retry", default_retry_window))

        ## now check the otp for each challenge

        for ch in challenges:
            challenge = {}

            ##  preserve transaction context, so we could use this in the status callback
            self.transId = ch.get('transid', None)
            challenge['transid'] = self.transId
            challenge['session'] = ch.get('session', None)

            ## we saved the 'real' challenge in the data
            data = ch.get('data', None)
            if data is not None:
                challenge['challenge'] = data.get('challenge')
            elif 'challenge' in ch:
                ## handle explicit challenge requests
                challenge['challenge'] = ch.get('challenge')

            if challenge.get('challenge') is None:
                raise Exception('could not checkOtp due to missing challenge'
                                ' in request: %r' % ch)

            ret = ocraSuite.checkOtp(passw, counter, window, challenge, pin=ocraPin , options=options, timeshift=timeShift)
            log.debug('ret %r' % (ret))

            ## due to the assynchronous challenge verification of the checkOtp
            ## it might happen, that the found counter is lower than the given
            ## one. Thus we fix this here to deny assynchronous verification

            # we do not support retry checks anymore:
            # which means, that ret might be smaller than the actual counter
            if ocraSuite.T is None:
                if ret + retry_window < counter:
                    ret = -1

            if ret != -1:
                break

        if -1 == ret:
            ##  autosync: test if two consecutive challenges + it's counter match
            ret = self.autosync(ocraSuite, passw, challenge)


        return ret

    @log_with(log)
    def autosync(self, ocraSuite, passw, challenge):
        '''
        try to resync a token automaticaly, if a former and the current request failed

        :param  ocraSuite: the ocraSuite of the current Token
        :type  ocraSuite: ocra object
        :param  passw:
        '''
        res = -1

        autosync = False

        try:
            async = getFromConfig("AutoResync")
            if async is None:
                autosync = False
            elif "true" == async.lower():
                autosync = True
            elif "false" == async.lower():
                autosync = False
        except Exception as ex:
            log.error('autosync check undefined %r' % (ex))
            return res

        ' if autosync is not enabled: do nothing '
        if False == autosync:
            return res

        ##
        ## AUTOSYNC starts here
        ##

        counter = self.token.getOtpCounter()
        syncWindow = self.token.getSyncWindow()
        if  ocraSuite.T is not None:
            syncWindow = syncWindow / 10


        ## set the ocra token pin
        ocraPin = ''
        if ocraSuite.P is not None:
            ocraPinObj = self.token.getUserPin()
            ocraPin = ocraPinObj.getKey()

            if ocraPin is None or len(ocraPin) == 0:
                ocraPin = ''

        timeShift = 0
        if  ocraSuite.T is not None:
            timeShift = int(self.getFromTokenInfo("timeShift", 0))

        #timeStepping    = int(ocraSuite.T)

        tinfo = self.getTokenInfo()

        ## autosync does only work, if we have a token info, where the last challenge and the last sync-counter is stored
        ## if no tokeninfo, we start with a autosync request, thus start the lookup in the sync window

        if tinfo.has_key('lChallenge') == False:
            ## run checkOtp, with sync window for the current challenge
            log.debug('initial sync')
            count_0 = -1
            try:
                otp0 = passw
                count_0 = ocraSuite.checkOtp(otp0, counter, syncWindow, challenge, pin=ocraPin, timeshift=timeShift)
            except Exception as ex:
                log.error(' error during autosync0 %r' % (ex))

            if count_0 != -1:
                tinfo['lChallenge'] = {'otpc' : count_0}
                self.setTokenInfo(tinfo)
                log.info('initial sync - success: %r' % (count_0))

            res = -1
            log.debug('initial sync done!')

        else:
            ## run checkOtp, with sync window for the current challenge
            log.debug('sync')
            count_1 = -1
            try:
                otp1 = passw
                count_1 = ocraSuite.checkOtp(otp1, counter, syncWindow, challenge, pin=ocraPin, timeshift=timeShift)
            except Exception as ex:
                log.error(' error during autosync1 %r' % (ex))

            if count_1 == -1:
                del tinfo['lChallenge']
                self.setTokenInfo(tinfo)
                log.info('sync failed! Not a valid pass in scope (%r)' % (otp1))
                res = -1
            else:
                ## run checkOtp, with sync window for the old challenge
                lChallange = tinfo.get('lChallenge')
                count_0 = lChallange.get('otpc')

                if ocraSuite.C is not None:
                    ##  sync the counter based ocra token
                    if count_1 - count_0 < 2:
                        self.setOtpCount(count_1)
                        res = count_1

                if ocraSuite.T is not None:
                    ##  sync the timebased ocra token
                    if count_1 - count_0 < ocraSuite.T * 2 :
                        ## calc the new timeshift !
                        log.debug("the counter %r matches: %r" %
                                  (count_1, datetime.datetime.fromtimestamp(count_1)))

                        currenttime = int(time.time())
                        new_shift = (count_1 - currenttime)

                        tinfo['timeShift'] = new_shift
                        self.setOtpCount(count_1)
                        res = count_1

                ##  if we came here, the old challenge is not required anymore
                del tinfo['lChallenge']
                self.setTokenInfo(tinfo)

        return res




    @log_with(log)
    def statusValidationFail(self):
        '''
        statusValidationFail - callback to enable a status change,

        will be called if the token verification has failed

        :return - nothing

        '''
        challenge = None

        if self.transId == 0 :
            return
        try:

            challenges = get_challenges(self.getSerial(), transid=self.transId)
            if len(challenges) == 1:
                challenge = challenges[0]
                challenge.setTanStatus(received=True, valid=False)


            ##  still in rollout state??
            rolloutState = self.getFromTokenInfo('rollout', '0')

            if rolloutState == '1':
                log.info('rollout state 1 for token %r not completed' % (self.getSerial()))

            elif rolloutState == '2':
                if challenge.received_count >= int(getFromConfig("OcraMaxChallengeRequests", '3')):
                    ##  after 3 fails in rollout state 2 - reset to rescan
                    self.addToTokenInfo('rollout', '1')
                    log.info('rollout for token %r reset to phase 1:' % (self.getSerial()))

                log.info('rollout for token %r not completed' % (self.getSerial()))

        except Exception as ex:
            log.error('Error during validation finalisation for token %r :%r' % (self.getSerial(), ex))
            log.error(traceback.format_exc())
            raise Exception(ex)

        finally:
            if challenge is not None:
                challenge.save()

        return

    @log_with(log)
    def statusValidationSuccess(self):
        '''
        statusValidationSuccess - callback to enable a status change,

        remark: will be called if the token has been succesfull verified

        :return: - nothing

        '''
        if self.transId == 0 :
            return

        challenges = get_challenges(self.getSerial(), transid=self.transId)
        if len(challenges) == 1:
            challenge = challenges[0]
            challenge.setTanStatus(True, True)
            challenge.save()

        ##  still in rollout state??
        rolloutState = self.getFromTokenInfo('rollout', '0')

        if rolloutState == '2':
            t_info = self.getTokenInfo()
            if t_info.has_key('rollout'):
                del t_info['rollout']
            if t_info.has_key('sharedSecret'):
                del t_info['sharedSecret']
            if t_info.has_key('nonce'):
                del t_info['nonce']
            self.setTokenInfo(t_info)

            log.info('rollout for token %r completed' % (self.getSerial()))

        elif rolloutState == '1':
            raise Exception('unable to complete the rollout ')

        return

    @log_with(log)
    def resync(self, otp1, otp2, options=None):
        '''
        - for the resync to work, we take the last two transactions and their challenges
        - for each challenge, we search forward the sync window length

        '''
        ret = False
        challenges = []

        ## the challenges are orderd, the first one is the newest
        challenges = get_challenges(self.getSerial())

        ##  check if there are enough challenges around
        if len(challenges) < 2:
            return False

        challenge1 = {}
        challenge2 = {}

        if options is None:

            ## the newer one
            ch1 = challenges[0]
            challenge1['challenge'] = ch1.get('data').get('challenge')
            challenge1['transid'] = ch1.get('transid')
            challenge1['session'] = ch1.get('session')
            challenge1['id'] = ch1.get('id')


            ## the elder one
            ch2 = challenges[0]
            challenge2['challenge'] = ch2.get('data').get('challenge')
            challenge2['transid'] = ch2.get('transid')
            challenge2['session'] = ch2.get('session')
            challenge2['id'] = ch2.get('id')

        else:
            if options.has_key('challenge1'):
                challenge1['challenge'] = options.get('challenge1')
            if options.has_key('challenge2'):
                challenge2['challenge'] = options.get('challenge2')


        if len(challenge1) == 0 or len(challenge2) == 0:
            error = "No challeges found!"
            log.error(error)
            raise Exception('[Ocra2TokenClass:resync] %s' % (error))



        secretHOtp = self.token.getHOtpKey()
        ocraSuite = OcraSuite(self.getOcraSuiteSuite(), secretHOtp)

        syncWindow = self.token.getSyncWindow()
        if  ocraSuite.T is not None:
            syncWindow = syncWindow / 10

        counter = self.token.getOtpCounter()

        ## set the ocra token pin
        ocraPin = ''
        if ocraSuite.P is not None:
            ocraPinObj = self.token.getUserPin()
            ocraPin = ocraPinObj.getKey()

            if ocraPin is None or len(ocraPin) == 0:
                ocraPin = ''

        timeShift = 0
        if  ocraSuite.T is not None:
            timeShift = int(self.getFromTokenInfo("timeShift", 0))

        try:

            count_1 = ocraSuite.checkOtp(otp1, counter, syncWindow, challenge1, pin=ocraPin, timeshift=timeShift)
            if count_1 == -1:
                log.info('lookup for first otp value failed!')
                ret = False
            else:
                count_2 = ocraSuite.checkOtp(otp2, counter, syncWindow, challenge2, pin=ocraPin, timeshift=timeShift)
                if count_2 == -1:
                    log.info('lookup for second otp value failed!')
                    ret = False
                else:
                    if ocraSuite.C is not None:
                        if count_1 + 1 == count_2:
                            self.setOtpCount(count_2)
                            ret = True

                    if  ocraSuite.T is not None:
                        if count_1 - count_2 <= ocraSuite.T * 2:
                            ##  callculate the timeshift
                            date = datetime.datetime.fromtimestamp(count_2)
                            log.info('syncing token to new timestamp: %r' % (date))

                            now = datetime.datetime.now()
                            stime = now.strftime("%s")
                            timeShift = count_2 - int(stime)
                            self.addToTokenInfo('timeShift', timeShift)
                            ret = True

        except Exception as ex:
            log.error('unknown error: %r' % (ex))
            raise Exception('[Ocra2TokenClass:resync] unknown error: %s' % (ex))

        return ret


    @log_with(log)
    def getStatus(self, transactionId):
        '''
        getStatus - assembles the status of a transaction / challenge in a dict

        {   "serial": SERIENNUMMER1,
            "transactionid": TRANSACTIONID1,
            "received_tan": true,
            "valid_tan": true,
            "failcount": 0
        }

        :param transactionId:    the transaction / challenge id
        :type transactionId:    string

        :return:    status dict
        :rtype:       dict
        '''
        statusDict = {}
        challenge = get_challenges(self.getSerial(), transid=transactionId)
        if challenge is not None:
            statusDict['serial'] = challenge.tokenserial
            statusDict['transactionid'] = challenge.transid
            statusDict['received_tan'] = challenge.received_tan
            statusDict['valid_tan'] = challenge.valid_tan
            statusDict['failcount'] = self.getFailCount()
            statusDict['id'] = challenge.id
            statusDict['timestamp'] = unicode(challenge.timestamp)
            statusDict['active'] = unicode(self.isActive())

        return statusDict


    @log_with(log)
    def getInitDetail(self, params , user=None):
        '''
        to complete the token normalisation, the response of the initialiastion
        should be build by the token specific method, the getInitDetails
        '''
        response_detail = {}

        info = self.getInfo()
        response_detail.update(info)

        ocra_url = info.get('app_import')
        response_detail["ocraurl"] = {
               "description" : _("URL for OCRA token"),
               "value" : ocra_url,
               "img"   : create_img(ocra_url, width=250)}

        return response_detail

    @log_with(log)
    def getQRImageData(self, response_detail):
        '''
        '''
        url = None
        hparam = {}

        if response_detail is not None:
            if 'ocraurl' in response_detail:
                url = response_detail.get('ocraurl', {}).get("value", "")
                hparam['alt'] = url
            if 'data' in response_detail:
                url = response_detail.get('data')
                hparam['alt'] = url

        return url, hparam


#eof###########################################################################




# -*- coding: utf-8 -*-
#
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  2014-10-04 Cornelius Kölbel <cornelius@privacyidea.org>
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
Description:  functional tests of ldap resolver
'''
import logging

from privacyidea.tests import TestController, url
from privacyidea.lib.resolvers.UserIdResolver import getResolverClass

log = logging.getLogger(__name__)


class TestSQLResolver(TestController):

    def _setup_sql_realm(self):
        parameters = {"name": "sqlres",
                      "type": "sqlresolver",
                      'Driver': 'sqlite',
                      'Server': '/privacyidea/tests/testdata/',
                      'Database': "testuser.sqlite",
                      'Table': 'users',
                      'Map': '{ "username": "username", \
                        "userid" : "id", \
                        "email" : "email", \
                        "surname" : "name", \
                        "givenname" : "givenname", \
                        "password" : "password", \
                        "phone": "phone", \
                        "mobile": "mobile"}'
                      }
        response = self.app.get(url(controller='system',
                                    action='setResolver'),
                                params=parameters)
        print response
        assert '"status": true' in response
        assert '"value": true' in response
        
        parameters = {"realm": "sqlrealm",
                      "resolvers": str("privacyidea.lib.resolvers."
                                       "SQLIdResolver."
                                       "IdResolver.sqlres")}
        response = self.app.get(url(controller='system',
                                    action='setRealm'),
                                params=parameters)
        print response
        assert '"status": true' in response
        assert '"value": true' in response
        
    def _cleanup_sql_realm(self):
                                
        response = self.app.get(url(controller='system',
                                    action='delRealm'),
                                params={'realm': 'sqlrealm'})
        print response
        assert '"result": true' in response
        
        response = self.app.get(url(controller='system',
                                    action='delResolver'),
                                params={'resolver': 'sqlres'})
        print response
        assert '"value": true' in response

    def test_01_standalone(self):
        '''
        Testing standalone functionality of SQL resolver
        '''
        y = getResolverClass("SQLIdResolver", "IdResolver")()
    
        y.loadConfig({'privacyidea.sqlresolver.Driver': 'sqlite',
                      'privacyidea.sqlresolver.Server': '/privacyidea/'
                      'tests/testdata/',
                      'privacyidea.sqlresolver.Database': "testuser.sqlite",
                      'privacyidea.sqlresolver.Table': 'users',
                      'privacyidea.sqlresolver.Encoding': "utf-8",
                      'privacyidea.sqlresolver.Map': '{ "username": "username", \
                          "userid" : "id", \
                          "email" : "email", \
                          "surname" : "name", \
                          "givenname" : "givenname", \
                          "password" : "password", \
                          "phone": "phone", \
                          "mobile": "mobile"}',
                      },
                     "")

        result = y.getUserList()
        print result
        assert len(result) == 5
        assert result[0].get("username") == "user1"
        assert result[1].get("username") == "fred"
        assert result[2].get("username") == "cornelius"
        
        user = "cornelius"
        UserId = y.getUserId(user)
        print UserId
        assert UserId == 3
        
        result = y.getUserInfo(0)
        print result
        assert {} == result
        
        user1 = {'username': u'user1',
                 'password': u'{SSHA512}zMpVSq1S/iJXpTYciIxssmDi+0iUpBZFNPfWh'
                 '0W9rnZFNvzQekuAhPEBEjEFqGEFsITfzo+LB0HTwcudZLcWm4meOzAn53kV',
                 'surname': u'dampf',
                 'mobile': u'+49 151 123454656',
                 'email': u'user1@testdomain.com',
                 'phone': u'030 123454566',
                 'givenname': u'hans',
                 'id': 1}
        result = y.getUserInfo(1)
        print result
        assert result == user1
        
        # {'username': u'cornelius', 'id': 3}
        result = y.getUserInfo(3)
        print result
        assert result.get("email") == "cornelius@privacyidea.org"
        
        # check for umlaut
        result = y.getUserInfo(3)
        print result
        assert result.get("surname") == u'K\xf6lbel'
        
        result = y.getUsername(3)
        print result
        assert "cornelius" in result
        
    def test_02_check_passwords(self):
        '''
        Checking SQL passwords
        '''
        '''
        SHA256 of "dunno"
        772cb52221f19104310cd2f549f5131fbfd34e0f4de7590c87b1d73175812607
        '''
        y = getResolverClass("SQLIdResolver", "IdResolver")()
    
        y.loadConfig({'privacyidea.sqlresolver.Driver': 'sqlite',
                      'privacyidea.sqlresolver.Server': '/privacyidea/'
                      'tests/testdata/',
                      'privacyidea.sqlresolver.Database': "testuser.sqlite",
                      'privacyidea.sqlresolver.Table': 'users',
                      'privacyidea.sqlresolver.Encoding': "utf-8",
                      'privacyidea.sqlresolver.Map': '{ "username": "username", \
                          "userid" : "id", \
                          "email" : "email", \
                          "surname" : "name", \
                          "givenname" : "givenname", \
                          "password" : "password", \
                          "phone": "phone", \
                          "mobile": "mobile"}',
                      },
                     "")
        
        result = y.checkPass(3, "dunno")
        print result
        assert result is True
        
        '''
        SHA1 base64 encoded of "dunno"
        Lg8DuLoXOwvPkMABDprnaTp0JOA=
        '''
        result = y.checkPass(2, "dunno")
        print result
        assert result is True
        
        result = y.checkPass(1, "dunno")
        print result
        assert result is True
        
        result = y.checkPass(4, "dunno")
        print result
        assert result is True
        
        result = y.checkPass(5, "dunno")
        print result
        assert result is True
        
    def test_03_testconnection(self):
        '''
        Testing standalone functionality of SQL resolver
        '''
        y = getResolverClass("SQLIdResolver", "IdResolver")()
    
        result = y.testconnection({'Driver': 'sqlite',
                                   'Server': '/privacyidea/tests/testdata/',
                                   'Database': "testuser.sqlite",
                                   'Table': 'users',
                                   'Encoding': "utf-8",
                                   'Map': '{ "username": "username", \
                                      "userid" : "id", \
                                      "email" : "email", \
                                      "surname" : "name", \
                                      "givenname" : "givenname", \
                                      "password" : "password", \
                                      "phone": "phone", \
                                      "mobile": "mobile"}',
                                   })
        print result
        assert result[0] == 5
        assert 'Found 5 users.' in result[1]
        
    def test_04_authentication(self):
        '''
        test auth with SQL resolver
        '''
        self._setup_sql_realm()
        parameters = {"serial": "sqltest1",
                      "user": "cornelius@sqlrealm",
                      "pin": "pin",
                      "type": "spass",
                      "description": "SQLTestToken",
                      }

        response = self.app.get(url(controller='admin',
                                    action='init'),
                                params=parameters)
        print response
        assert '"value": true' in response
        
        response = self.app.get(url(controller='validate',
                                    action='check'),
                                params={'user': 'cornelius@sqlrealm',
                                        'pass': 'pin'})
        print response
        assert '"value": true' in response

        # cleanup
        response = self.app.get(url(controller='admin',
                                    action='remove'),
                                params={'serial': 'sqltest1'})
        print response
        assert '"value": 1' in response
        
        self._cleanup_sql_realm()
        


#!/usr/bin/env python
#
# phpass version: 0.3 / genuine.
#
# Placed in public domain
#
# source: https://github.com/exavolt/python-phpass
#

# CHECK: use pyDES instead of the native crypt module?

import os
import time
import hashlib
import crypt


try:
    import bcrypt
    _bcrypt_hashpw = bcrypt.hashpw
except ImportError:
    _bcrypt_hashpw = None

# On App Engine, this function is not available.
if hasattr(os, 'getpid'):
    _pid = os.getpid()
else:
    # Fake PID
    from privacyidea.lib.crypto import urandom
    _pid = urandom.randint(0, 100000)


class PasswordHash:

    def __init__(self, iteration_count_log2=8, portable_hashes=True,
                 algorithm=''):
        alg = algorithm.lower()
        if (alg == 'blowfish' or alg == 'bcrypt') and _bcrypt_hashpw is None:
            raise NotImplementedError('The bcrypt module is required')
        self.itoa64 = \
            './0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'
        if iteration_count_log2 < 4 or iteration_count_log2 > 31:
            iteration_count_log2 = 8
        self.iteration_count_log2 = iteration_count_log2
        self.portable_hashes = portable_hashes
        self.algorithm = algorithm
        self.random_state = '%r%r' % (time.time(), _pid)

    def get_random_bytes(self, count):
        outp = ''
        try:
            outp = os.urandom(count)
        except:
            pass
        if len(outp) < count:
            outp = ''
            rem = count
            while rem > 0:
                self.random_state = hashlib.md5(str(time.time())
                    + self.random_state).hexdigest()
                outp += hashlib.md5(self.random_state).digest()
                rem -= 1
            outp = outp[:count]
        return outp

    def encode64(self, inp, count):
        outp = ''
        cur = 0
        while cur < count:
            value = ord(inp[cur])
            cur += 1
            outp += self.itoa64[value & 0x3f]
            if cur < count:
                value |= (ord(inp[cur]) << 8)
            outp += self.itoa64[(value >> 6) & 0x3f]
            if cur >= count:
                break
            cur += 1
            if cur < count:
                value |= (ord(inp[cur]) << 16)
            outp += self.itoa64[(value >> 12) & 0x3f]
            if cur >= count:
                break
            cur += 1
            outp += self.itoa64[(value >> 18) & 0x3f]
        return outp

    def gensalt_private(self, inp):
        outp = '$P$'
        outp += self.itoa64[min([self.iteration_count_log2 + 5, 30])]
        outp += self.encode64(inp, 6)
        return outp

    def crypt_private(self, pw, setting):
        outp = '*0'
        if setting.startswith(outp):
            outp = '*1'
        if not setting.startswith('$P$') and not setting.startswith('$H$'):
            return outp
        count_log2 = self.itoa64.find(setting[3])
        if count_log2 < 7 or count_log2 > 30:
            return outp
        count = 1 << count_log2
        salt = setting[4:12]
        if len(salt) != 8:
            return outp
        if not isinstance(pw, str):
            pw = pw.encode('utf-8')
        hx = hashlib.md5(salt + pw).digest()
        while count:
            hx = hashlib.md5(hx + pw).digest()
            count -= 1
        return setting[:12] + self.encode64(hx, 16)

    def gensalt_extended(self, inp):
        count_log2 = min([self.iteration_count_log2 + 8, 24])
        count = (1 << count_log2) - 1
        outp = '_'
        outp += self.itoa64[count & 0x3f]
        outp += self.itoa64[(count >> 6) & 0x3f]
        outp += self.itoa64[(count >> 12) & 0x3f]
        outp += self.itoa64[(count >> 18) & 0x3f]
        outp += self.encode64(inp, 3)
        return outp

    def gensalt_blowfish(self, inp):
        itoa64 = \
            './ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789'
        outp = '$2a$'
        outp += chr(ord('0') + self.iteration_count_log2 / 10)
        outp += chr(ord('0') + self.iteration_count_log2 % 10)
        outp += '$'
        cur = 0
        while True:
            c1 = ord(inp[cur])
            cur += 1
            outp += itoa64[c1 >> 2]
            c1 = (c1 & 0x03) << 4
            if cur >= 16:
                outp += itoa64[c1]
                break
            c2 = ord(inp[cur])
            cur += 1
            c1 |= c2 >> 4
            outp += itoa64[c1]
            c1 = (c2 & 0x0f) << 2
            c2 = ord(inp[cur])
            cur += 1
            c1 |= c2 >> 6
            outp += itoa64[c1]
            outp += itoa64[c2 & 0x3f]
        return outp

    def hash_password(self, pw):
        rnd = ''
        alg = self.algorithm.lower()
        if (not alg or alg == 'blowfish' or alg == 'bcrypt') \
             and not self.portable_hashes:
            if _bcrypt_hashpw is None:
                if (alg == 'blowfish' or alg == 'bcrypt'):
                    raise NotImplementedError('The bcrypt module is required')
            else:
                rnd = self.get_random_bytes(16)
                salt = self.gensalt_blowfish(rnd)
                hx = _bcrypt_hashpw(pw, salt)
                if len(hx) == 60:
                    return hx
        if (not alg or alg == 'ext-des') and not self.portable_hashes:
            if len(rnd) < 3:
                rnd = self.get_random_bytes(3)
            hx = crypt.crypt(pw, self.gensalt_extended(rnd))
            if len(hx) == 20:
                return hx
        if len(rnd) < 6:
            rnd = self.get_random_bytes(6)
        hx = self.crypt_private(pw, self.gensalt_private(rnd))
        if len(hx) == 34:
            return hx
        return '*'

    def check_password(self, pw, stored_hash):
        # This part is different with the original PHP
        if stored_hash.startswith('$2a$'):
            # bcrypt
            if _bcrypt_hashpw is None:
                raise NotImplementedError('The bcrypt module is required')
            hx = _bcrypt_hashpw(pw, stored_hash)
        elif stored_hash.startswith('_'):
            # ext-des
            hx = crypt.crypt(pw, stored_hash)
        else:
            # portable hash
            hx = self.crypt_private(pw, stored_hash)
        return hx == stored_hash


if __name__ == "__main__":  # pragma: no cover
    import getpass
    while True:
        pw = getpass.getpass()
        pw2 = getpass.getpass('Retype password: ')
        if pw == pw2:
            break
        print "Both passwords must be the same"
    t_hasher = PasswordHash(8, True)
    print "Password hash: " + t_hasher.hash_password(pw)




# -*- coding: utf-8 -*-
#
#    privacyIDEA is a fork of LinOTP
#    May 28, 2014 Cornelius Kölbel
#    E-mail: info@privacyidea.org
#    Contact: www.privacyidea.org
#
#    Copyright (C) LinOTP: 2010 - 2014 LSE Leading Security Experts GmbH
#
#    This program is free software: you can redistribute it and/or
#    modify it under the terms of the GNU Affero General Public
#    License, version 3, as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the
#               GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#

"""
                This is a dummy class for testing
"""
from privacyidea.smsprovider.SMSProvider import ISMSProvider


import logging
log = logging.getLogger(__name__)


class TestSMSProvider(ISMSProvider):
    def __init__(self):
        self.config = {}

    '''
      submitMessage()
      - send out a message to a phone

    '''

    def submitMessage(self, phone, message):
        return

    def getParameters(self, message, phone):
        return

    def loadConfig(self, configDict):
        self.config = configDict




# -*- coding: utf-8 -*-
#
#    privacyIDEA Account test suite
# 
#    Copyright (C)  2014 Cornelius Kölbel, cornelius@privacyidea.org
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
log = logging.getLogger(__name__)
import unittest
import os
from privacyidea.lib.freeradiusparser import ClientConfParser
from privacyidea.lib.freeradiusparser import UserConfParser

CLIENT_CONF = """# This is a client.conf for freeradius
client localhost {
        ipaddr          = 127.0.0.1
        secret          = testing123
        shortname       = localhost
        nastype         = other
}

client private-network-1 {
        ipaddr          = 192.168.0.0
        netmask         = 24
        secret          = testing123-1
        shortname       = private-network-1
}
"""

CLIENT_CONF_ORIG="""#
# clients.conf - client configuration directives
#
#######################################################################

#######################################################################
#
#  Definition of a RADIUS client (usually a NAS).
#
#  The information given here over rides anything given in the
#  'clients' file, or in the 'naslist' file.  The configuration here
#  contains all of the information from those two files, and allows
#  for more configuration items.
#
#  The "shortname" is be used for logging.  The "nastype", "login" and
#  "password" fields are mainly used for checkrad and are optional.
#

#
#  Defines a RADIUS client.  The format is 'client [hostname|ip-address]'
#
#  '127.0.0.1' is another name for 'localhost'.  It is enabled by default,
#  to allow testing of the server after an initial installation.  If you
#  are not going to be permitting RADIUS queries from localhost, we suggest
#  that you delete, or comment out, this entry.
#
client 127.0.0.1 {
    #
    #  The shared secret use to "encrypt" and "sign" packets between
    #  the NAS and FreeRADIUS.  You MUST change this secret from the
    #  default, otherwise it's not a secret any more!
    #
    #  The secret can be any string, up to 31 characters in length.
    #
    secret        = testing123

    #
    #  The short name is used as an alias for the fully qualified
    #  domain name, or the IP address.
    #
    shortname    = localhost

    #
    # the following three fields are optional, but may be used by
    # checkrad.pl for simultaneous use checks
    #

    #
    # The nastype tells 'checkrad.pl' which NAS-specific method to
    #  use to query the NAS for simultaneous use.
    #
    #  Permitted NAS types are:
    #
    #    cisco
    #    computone
    #    livingston
    #    max40xx
    #    multitech
    #    netserver
    #    pathras
    #    patton
    #    portslave
    #    tc
    #    usrhiper
    #    other        # for all other types

    #
    nastype     = other    # localhost isn't usually a NAS...

    #
    #  The following two configurations are for future use.
    #  The 'naspasswd' file is currently used to store the NAS
    #  login name and password, which is used by checkrad.pl
    #  when querying the NAS for simultaneous use.
    #
#    login       = !root
#    password    = someadminpas
}

#client some.host.org {
#    secret        = testing123
#    shortname    = localhost
#}

#
#  You can now specify one secret for a network of clients.
#  When a client request comes in, the BEST match is chosen.
#  i.e. The entry from the smallest possible network.
#
#client 192.168.0.0/24 {
#    secret        = testing123-1
#    shortname    = private-network-1
#}
#
#client 192.168.0.0/16 {
#    secret        = testing123-2
#    shortname    = private-network-2
#}


#client 10.10.10.10 {
#    # secret and password are mapped through the "secrets" file.
#    secret      = testing123
#    shortname   = liv1
#       # the following three fields are optional, but may be used by
#       # checkrad.pl for simultaneous usage checks
#    nastype     = livingston
#    login       = !root
#    password    = someadminpas
#}


"""

FILEOUTPUT_ORIG = """# File parsed and saved by privacyidea.

client 127.0.0.1 {
    secret = testing123
    shortname = localhost
    nastype = other
}

"""

USER_CONF_A = """
DEFAULT Auth-Type := perl
"""

USER_CONF_B = """
administrator Cleartext-Password := "secret"
"""

USER_CONF_C = """
user somekey == value
"""
USER_CONF_D = """
DEFAULT    Hint == "SLIP"
    Framed-Protocol = SLIP
"""

USER_CONF_E = """
DEFAULT    Hint == "SLIP"
    Framed-Protocol = SLIP,
    Something = Else
"""

USER_CONF_1 = """
#
#    Please read the documentation file ../doc/processing_users_file,
#    or 'man 5 users' (after installing the server) for more information.
#
#    This file contains authentication security and configuration
#    information for each user.  Accounting requests are NOT processed
#    through this file.  Instead, see 'acct_users', in this directory.
#
#    The first field is the user's name and can be up to
#    253 characters in length.  This is followed (on the same line) with
#    the list of authentication requirements for that user.  This can
#    include password, comm server name, comm server port number, protocol
#    type (perhaps set by the "hints" file), and huntgroup name (set by
#    the "huntgroups" file).
#
#    If you are not sure why a particular reply is being sent by the
#    server, then run the server in debugging mode (radiusd -X), and
#    you will see which entries in this file are matched.
#
#    When an authentication request is received from the comm server,
#    these values are tested. Only the first match is used unless the
#    "Fall-Through" variable is set to "Yes".
#
#    A special user named "DEFAULT" matches on all usernames.
#    You can have several DEFAULT entries. All entries are processed
#    in the order they appear in this file. The first entry that
#    matches the login-request will stop processing unless you use
#    the Fall-Through variable.
#
#    If you use the database support to turn this file into a .db or .dbm
#    file, the DEFAULT entries _have_ to be at the end of this file and
#    you can't have multiple entries for one username.
#
#    Indented (with the tab character) lines following the first
#    line indicate the configuration values to be passed back to
#    the comm server to allow the initiation of a user session.
#    This can include things like the PPP configuration values
#    or the host to log the user onto.
#
#    You can include another `users' file with `$INCLUDE users.other'
#

#
#    For a list of RADIUS attributes, and links to their definitions,
#    see:
#
#    http://www.freeradius.org/rfc/attributes.html
#

#
# Deny access for a specific user.  Note that this entry MUST
# be before any other 'Auth-Type' attribute which results in the user
# being authenticated.
#
# Note that there is NO 'Fall-Through' attribute, so the user will not
# be given any additional resources.
#
#lameuser    Auth-Type := Reject
#        Reply-Message = "Your account has been disabled."

#
# Deny access for a group of users.
#
# Note that there is NO 'Fall-Through' attribute, so the user will not
# be given any additional resources.
#
#DEFAULT    Group == "disabled", Auth-Type := Reject
#        Reply-Message = "Your account has been disabled."
#

#
# This is a complete entry for "steve". Note that there is no Fall-Through
# entry so that no DEFAULT entry will be used, and the user will NOT
# get any attributes in addition to the ones listed here.
#
#steve    Cleartext-Password := "testing"
#    Service-Type = Framed-User,
#    Framed-Protocol = PPP,
#    Framed-IP-Address = 172.16.3.33,
#    Framed-IP-Netmask = 255.255.255.0,
#    Framed-Routing = Broadcast-Listen,
#    Framed-Filter-Id = "std.ppp",
#    Framed-MTU = 1500,
#    Framed-Compression = Van-Jacobsen-TCP-IP

#
# This is an entry for a user with a space in their name.
# Note the double quotes surrounding the name.
#
#"John Doe"    Cleartext-Password := "hello"
#        Reply-Message = "Hello, %{User-Name}"

#
# Dial user back and telnet to the default host for that port
#
#Deg    Cleartext-Password := "ge55ged"
#    Service-Type = Callback-Login-User,
#    Login-IP-Host = 0.0.0.0,
#    Callback-Number = "9,5551212",
#    Login-Service = Telnet,
#    Login-TCP-Port = Telnet

#
# Another complete entry. After the user "dialbk" has logged in, the
# connection will be broken and the user will be dialed back after which
# he will get a connection to the host "timeshare1".
#
#dialbk    Cleartext-Password := "callme"
#    Service-Type = Callback-Login-User,
#    Login-IP-Host = timeshare1,
#    Login-Service = PortMaster,
#    Callback-Number = "9,1-800-555-1212"

#
# user "swilson" will only get a static IP number if he logs in with
# a framed protocol on a terminal server in Alphen (see the huntgroups file).
#
# Note that by setting "Fall-Through", other attributes will be added from
# the following DEFAULT entries
#
#swilson    Service-Type == Framed-User, Huntgroup-Name == "alphen"
#        Framed-IP-Address = 192.168.1.65,
#        Fall-Through = Yes

#
# If the user logs in as 'username.shell', then authenticate them
# using the default method, give them shell access, and stop processing
# the rest of the file.
#
#DEFAULT    Suffix == ".shell"
#        Service-Type = Login-User,
#        Login-Service = Telnet,
#        Login-IP-Host = your.shell.machine


#
# The rest of this file contains the several DEFAULT entries.
# DEFAULT entries match with all login names.
# Note that DEFAULT entries can also Fall-Through (see first entry).
# A name-value pair from a DEFAULT entry will _NEVER_ override
# an already existing name-value pair.
#

#
# Set up different IP address pools for the terminal servers.
# Note that the "+" behind the IP address means that this is the "base"
# IP address. The Port-Id (S0, S1 etc) will be added to it.
#
#DEFAULT    Service-Type == Framed-User, Huntgroup-Name == "alphen"
#        Framed-IP-Address = 192.168.1.32+,
#        Fall-Through = Yes

#DEFAULT    Service-Type == Framed-User, Huntgroup-Name == "delft"
#        Framed-IP-Address = 192.168.2.32+,
#        Fall-Through = Yes

#
# Sample defaults for all framed connections.
#
#DEFAULT    Service-Type == Framed-User
#    Framed-IP-Address = 255.255.255.254,
#    Framed-MTU = 576,
#    Service-Type = Framed-User,
#    Fall-Through = Yes

#
# Default for PPP: dynamic IP address, PPP mode, VJ-compression.
# NOTE: we do not use Hint = "PPP", since PPP might also be auto-detected
#    by the terminal server in which case there may not be a "P" suffix.
#    The terminal server sends "Framed-Protocol = PPP" for auto PPP.
#
DEFAULT    Framed-Protocol == PPP
    Framed-Protocol = PPP,
    Framed-Compression = Van-Jacobson-TCP-IP

#
# Default for CSLIP: dynamic IP address, SLIP mode, VJ-compression.
#
DEFAULT    Hint == "CSLIP"
    Framed-Protocol = SLIP,
    Framed-Compression = Van-Jacobson-TCP-IP

#
# Default for SLIP: dynamic IP address, SLIP mode.
#
DEFAULT    Hint == "SLIP"
    Framed-Protocol = SLIP

#
# Last default: rlogin to our main server.
#
#DEFAULT
#    Service-Type = Login-User,
#    Login-Service = Rlogin,
#    Login-IP-Host = shellbox.ispdomain.com

# #
# # Last default: shell on the local terminal server.
# #
# DEFAULT
#     Service-Type = Administrative-User

# On no match, the user is denied access.
"""

FILEOUTPUT_USER_ORIG = """# File parsed and saved by privacyidea.

DEFAULT Framed-Protocol == PPP
\tFramed-Protocol = PPP,
\tFramed-Compression = Van-Jacobson-TCP-IP

DEFAULT Hint == "CSLIP"
\tFramed-Protocol = SLIP,
\tFramed-Compression = Van-Jacobson-TCP-IP

DEFAULT Hint == "SLIP"
\tFramed-Protocol = SLIP

"""


class TestFreeRADIUSParser(unittest.TestCase):

    def setUp(self):
        pass
    
    def test_clients_conf_simple(self):
        CP = ClientConfParser(content=CLIENT_CONF)
        config = CP.get_dict()
        print config
        self.assertTrue("localhost" in config)
        self.assertTrue("private-network-1" in config)
        self.assertEqual(config.get("private-network-1").get("secret"),
                         "testing123-1")
        self.assertEqual(config.get("private-network-1").get("shortname"),
                         "private-network-1")
        
    def test_clients_conf_original(self):
        CP = ClientConfParser(content=CLIENT_CONF_ORIG)
        config = CP.get_dict()
        print config
        # {'127.0.0.1': {'secret': 'testing123', 'shortname': 'localhost',
        # 'nastype': 'other    '}}
        self.assertTrue("localhost" not in config)
        self.assertTrue("127.0.0.1" in config)
        self.assertEqual(config.get("127.0.0.1").get("secret"),
                         "testing123")
        self.assertEqual(config.get("127.0.0.1").get("nastype"),
                         "other")
        
        output = CP.format(config)
        print output
        self.assertEqual(output, FILEOUTPUT_ORIG)
        
    def test_save_file(self):
        tmpfile = "./tmp-output"
        CP = ClientConfParser(content=CLIENT_CONF_ORIG)
        config = CP.get_dict()
        print config
        CP.save(config, tmpfile)
        f = open(tmpfile, "r")
        output = f.read()
        f.close()
        os.unlink(tmpfile)
        print output

        self.assertEqual(output, FILEOUTPUT_ORIG)
        

class TestFreeRADIUSUsers(unittest.TestCase):
    
    def setUp(self):
        pass
    
    def test_users_basic(self):
        UP = UserConfParser(content=USER_CONF_A)
        config = UP.get()
        print config
        self.assertEqual(config[0][0], "DEFAULT")
        self.assertEqual(config[0][1], "Auth-Type")
        self.assertEqual(config[0][2], ":=")
        self.assertEqual(config[0][3], "perl")
        
    def test_users_password(self):
        UP = UserConfParser(content=USER_CONF_B)
        config = UP.get()
        print config
        self.assertEqual(config[0][0], "administrator")
        self.assertEqual(config[0][1], "Cleartext-Password")
        self.assertEqual(config[0][2], ":=")
        self.assertEqual(config[0][3], '"secret"')
        
    def test_operators(self):
        UP = UserConfParser(content=USER_CONF_C)
        config = UP.get()
        print config
        self.assertEqual(config[0][0], "user")
        self.assertEqual(config[0][1], "somekey")
        self.assertEqual(config[0][2], "==")
        self.assertEqual(config[0][3], 'value')

    def test_reply_items(self):
        UP = UserConfParser(content=USER_CONF_D)
        config = UP.get()
        print config
        # [['DEFAULT', 'Hint', '==', '"SLIP"',
        #  [['Framed-Protocol', '=', 'SLIP']]]
        # ]

        self.assertEqual(config[0][0], "DEFAULT")
        self.assertEqual(config[0][1], "Hint")
        self.assertEqual(config[0][2], "==")
        self.assertEqual(config[0][3], '"SLIP"')
        self.assertEqual(config[0][4][0][0], 'Framed-Protocol')
        self.assertEqual(config[0][4][0][1], '=')
        self.assertEqual(config[0][4][0][2], 'SLIP')
        
    def test_reply_items2(self):
        UP = UserConfParser(content=USER_CONF_E)
        config = UP.get()
        print config
        # [['DEFAULT', 'Hint', '==', '"SLIP"',
        #  [['Framed-Protocol', '=', 'SLIP']]]
        # ]

        self.assertEqual(config[0][0], "DEFAULT")
        self.assertEqual(config[0][1], "Hint")
        self.assertEqual(config[0][2], "==")
        self.assertEqual(config[0][3], '"SLIP"')
        self.assertEqual(config[0][4][0][0], 'Framed-Protocol')
        self.assertEqual(config[0][4][0][1], '=')
        self.assertEqual(config[0][4][0][2], 'SLIP')
        self.assertEqual(config[0][4][1][0], 'Something')
        self.assertEqual(config[0][4][1][1], '=')
        self.assertEqual(config[0][4][1][2], 'Else')
    
    def test_get_complete(self):
        UP = UserConfParser(content=USER_CONF_1)
        config = UP.get()
        print len(config)
        self.assertEqual(len(config), 3)
        print config[0]
        print config[1]
        print config[2]
        self.assertEqual(len(config[0][4]), 2)
        self.assertEqual(len(config[1][4]), 2)
        self.assertEqual(len(config[2][4]), 1)
        
    def test_save_file(self):
        tmpfile = "./tmp-output"
        UP = UserConfParser(content=USER_CONF_1)
        config = UP.get()
        print config
        UP.save(config, tmpfile)
        f = open(tmpfile, "r")
        output = f.read()
        f.close()
        os.unlink(tmpfile)
        print output

        self.assertEqual(output, FILEOUTPUT_USER_ORIG)
    
if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''     
  Description:  functional tests
                
  Dependencies: -

'''
import logging
from privacyidea.tests import TestController, url
import datetime

log = logging.getLogger(__name__)

class TestSystemController(TestController):


    ###############################################################################
    def test_setDefault(self):
        '''
        Testing setting default values
        '''

        """
            response = self.app.get(url_for(controller='page', action='view', id=1))
            get(url, params=None, headers=None, extra_environ=None, status=None, expect_errors=False)

        setDefault: parameters are\
            DefaultMaxFailCount\
            DefaultSyncWindow\
            DefaultCountWindow\
            DefaultOtpLen\
            DefaultResetFailCount\
        "


        """


        parameters = {
                      "DefaultMaxFailCount":"21",
                      "DefaultSyncWindow":"200",
                      "DefaultCountWindow" : "20",
                      "DefaultOtpLen" : "8",
                      "DefaultResetFailCount" : "False"
                      }
        response = self.app.get(url(controller='system', action='setDefault'), params=parameters)
        #log.debug("response %s\n",response)
        assert '"set DefaultSyncWindow": true' in response
        assert '"set DefaultMaxFailCount": true' in response
        assert '"set DefaultResetFailCount": true' in response
        assert '"set DefaultSyncWindow": true' in response
        assert '"set DefaultMaxFailCount": true' in response
        assert '"set DefaultCountWindow": true'in response



        parameters = {
                      "DefaultMaxFailCount":"10",
                      "DefaultSyncWindow":"1000",
                      "DefaultCountWindow" : "10",
                      "DefaultOtpLen" : "6",
                      "DefaultResetFailCount" : "True"
                      }

        response = self.app.get(url(controller='system', action='setDefault'), params=parameters)
        #log.info("response %s\n",response)
        assert '"set DefaultSyncWindow": true' in response
        assert '"set DefaultMaxFailCount": true' in response
        assert '"set DefaultResetFailCount": true' in response
        assert '"set DefaultSyncWindow": true' in response
        assert '"set DefaultMaxFailCount": true' in response
        assert '"set DefaultCountWindow": true'in response

    def test_001_resolvers(self):
        self.__deleteAllRealms__()
        parameters = {
                              "username":"root",
                     }

        response = self.app.get(url(controller='admin', action='userlist'), params=parameters)
        parameters = {
                      "username":"root",
                      "realm":"myMixRealm"
                      }

        response = self.app.get(url(controller='admin', action='userlist'), params=parameters)
        log.debug(response)


    def test_001_realms(self):

        response = self.app.get(url(controller='system', action='getRealms'))
        #log.info("response %s\n",response)

        ## set realms
        assert '"realmname": "mydefrealm"'in response
        assert '"realmname": "myotherrealm"'in response
        assert '"realmname": "mymixrealm"'in response

        ## now check for the different users in the different realms
        parameters = {
                      "username":"root",
                      "realm":"*"
                      }

        response = self.app.get(url(controller='admin', action='userlist'), params=parameters)
        #log.info("response %s\n",response)

        ## http://127.0.0.1:5001/admin/userlist?username=root
        ## # description: "root-def-passwd"


        assert '"privacyidea.lib.resolvers.PasswdIdResolver.IdResolver.myOtherRes"'in response
        assert '"privacyidea.lib.resolvers.PasswdIdResolver.IdResolver.myDefRes"'in response

        ## now check for the different users in the different realms
        parameters = {
                      "username":"root",
                      "realm":"myDefRealm"
                      }

        response = self.app.get(url(controller='admin', action='userlist'), params=parameters)
        #log.info("response %s\n",response)

        print response
        assert '"privacyidea.lib.resolvers.PasswdIdResolver.IdResolver.myDefRes"'in response
        assert '"root-def-passwd"'in response

        ## now check for the different users in the different realms
        parameters = {
                      "username":"root",
                      "realm":"myMixRealm"
                      }

        response = self.app.get(url(controller='admin', action='userlist'), params=parameters)
        #log.info("response %s\n",response)

        assert '"privacyidea.lib.resolvers.PasswdIdResolver.IdResolver.myOtherRes"'in response
        assert '"root-myDom-passwd"'in response

        assert '"privacyidea.lib.resolvers.PasswdIdResolver.IdResolver.myDefRes"'in response
        assert '"root-def-passwd"'in response


        ## now check for the different users in the different realms
        parameters = {
                      "username":"root",  ## check in default
                      }

        response = self.app.get(url(controller='admin', action='userlist'), params=parameters)
        #log.info("response %s\n",response)

        assert '"privacyidea.lib.resolvers.PasswdIdResolver.IdResolver.myDefRes"'in response
        assert '"root-def-passwd"'in response


        ## now set default to myDomain
        parameters = {
                      "realm":"myOtherRealm"
                      }

        response = self.app.get(url(controller='system', action='setDefaultRealm'), params=parameters)
        assert '"value": true'in response



        ## now check for the different users in the different realms
        parameters = {
                      "username":"root",  ## check in default
                      }

        response = self.app.get(url(controller='admin', action='userlist'), params=parameters)
        #log.info("response %s\n",response)
        assert '"privacyidea.lib.resolvers.PasswdIdResolver.IdResolver.myOtherRes"'in response
        assert '"root-myDom-passwd"'in response


        ## now delete the default realm
        parameters = {
                      "realm":"myOtherRealm",  ## check in default
                      }

        response = self.app.get(url(controller='system', action='delRealm'), params=parameters)
        #log.info("delRealm: %s ------\n%s" % (str(parameters), str(response)))
        assert '"delRealm": {'in response
        assert '"result": true'in response

        parameters = {
                      "realms":"*",
                      }

        response = self.app.get(url(controller='system', action='getRealms'))
        ## set realms
        assert '"realmname": "mydefrealm"'in response
        assert '"realmname": "myotherrealm"' not in response
        assert '"realmname": "mymixrealm"'in response


        ## now check for the different users in the different realms
        parameters = {
                      "username":"def",  ## check in default
                      }

        response = self.app.get(url(controller='admin', action='userlist'), params=parameters)
        #log.info("response %s\n",response)
        assert '"value": []'in response

        ## now check for the different users in the different realms
        parameters = {
                      "username":"def",  ## check in default
                      "realm":"myDefRealm"
                      }

        response = self.app.get(url(controller='admin', action='userlist'), params=parameters)
        #log.info("response %s\n",response)
        assert '"description": "def User,,,,"'in response


        ## now set default to myDomain
        parameters = {
                      "realm":"myDefRealm"
                      }

        response = self.app.get(url(controller='system', action='setDefaultRealm'), params=parameters)
        assert '"value": true'in response


        ## now check for the different users in the different realms
        parameters = {
                      "username":"def",  ## check in default
                      }

        response = self.app.get(url(controller='admin', action='userlist'), params=parameters)
        #log.info("response %s\n",response)
        assert '"description": "def User,,,,"'in response

        ## now set default to myDomain
        parameters = {
                      "realm":"myMixRealm"
                      }

        response = self.app.get(url(controller='system', action='setDefaultRealm'), params=parameters)
        assert '"value": true'in response


        ## now check for the different users in the different realms
        parameters = {
                      "username":"root",  ## check in default
                      }

        response = self.app.get(url(controller='admin', action='userlist'), params=parameters)
        #log.info("response %s\n",response)
        assert '"root-def-passwd"'in response
        assert '"root-myDom-passwd"'in response



        ## now set default to myDomain
        parameters = {
                      "realm":"myOtherRealm"
                      }

        response = self.app.get(url(controller='system', action='setDefaultRealm'), params=parameters)
        assert '"value": false'in response




        ## now check for the different users in the different realms
        parameters = {
                      "username":"def",  ## check in default
                      "resConf":"myDefRes"
                      }

        response = self.app.get(url(controller='admin', action='userlist'), params=parameters)
        #log.info("response %s\n",response)
        assert '"description": "def User,,,,"'in response



    def test_import_policy(self):

        policy_content = '''[resovler_ss1]
realm = realm2
client = None
user = lse_ad:
time = None
action = "webprovisionGOOGLE, "
scope = selfservice
[resovler_ss2]
realm = realm2
client = None
user = "local:, koelbel"
time = None
action = "webprovisionGOOGLEtime, assign, "
scope = selfservice
[ss1_maria]
realm = realm1
client = None
user = maria
time = None
action = "max_count_hotp=10, webprovisionGOOGLE, getotp, webprovisionOCRA, enrollYUBICO, "
scope = selfservice
[SMS]
realm = realm1
client = ""
user = ""
time = ""
action = smstext=The OTP value for <serial>: <otp>
scope = authentication
[ss1_raff]
realm = realm1
client = None
user = None
time = None
action = "webprovisionGOOGLE, max_count_hotp=5, getotp, assign, webprovisionOCRA, webprovisionOCRA, enrollSMS, enrollMOTP, setMOTPPIN, history"
scope = selfservice
[ocra]
realm = *
client = *
user = admin
time = None
action = "request, activationcode, status, "
scope = ocra
[ss1_ocra]
realm = realm1
client = None
user = None
time = None
action = "qrtanurl=https://localhost, "
scope = authentication
[gettoken]
realm = *
client = ""
user = *
time = ""
action = max_count_hotp=50
scope = gettoken
'''
        response = self.app.post(url(controller='system', action='importPolicy'),
                                 params={},
                                 upload_files=[("file", "savedPolicy.txt", policy_content)])
        print response
        assert '<status>True</status>' in response
        assert '<value>8</value>' in response

        # Now check the policies, that we imported...

        response = self.app.post(url(controller='system', action='getPolicy'),
                                 params={'selftest_admin' : 'superuser'})
        print response
        assert '"resovler_ss1": {' in response
        assert '"resovler_ss2": {' in response
        assert '"ss1_maria": {' in response
        assert '"SMS": {' in response
        assert '"ss1_raff": {' in response
        assert '"ocra": {' in response
        assert '"ss1_ocra": {' in response
        assert '"gettoken": {' in response

        # Now we try to upload with access policies.

        response = self.app.post(url(controller='system', action='setPolicy'),
                                 params={'selftest_admin' : 'superuser',
                                         'name' : 'superuser',
                                         'scope' : 'system',
                                         'action' : 'read,write',
                                         'realm' : '*',
                                         'user' : 'superuser'})
        print response
        assert '"setPolicy superuser":' in response

        response = self.app.post(url(controller='system', action='setPolicy'),
                                 params={'selftest_admin' : 'superuser',
                                         'name' : 'readsystem',
                                         'scope' : 'system',
                                         'action' : 'read',
                                         'realm' : '*',
                                         'user' : 'readadmin'})
        print response
        assert '"setPolicy readsystem":' in response

        # superuser is allowed to import

        response = self.app.post(url(controller='system', action='importPolicy'),
                                 params={'selftest_admin': 'superuser'},
                                 upload_files=[("file", "savedPolicy.txt", policy_content)])
        print response
        assert '<status>True</status>' in response
        assert '<value>8</value>' in response

        # readadmin is not allowed to import

        response = self.app.post(url(controller='system', action='importPolicy'),
                                 params={'selftest_admin': 'readadmin'},
                                 upload_files=[("file", "savedPolicy.txt", policy_content)])
        print response
        assert 'Policy check failed. You are not allowed to write system config' in response

    def test_set_default(self):
        '''
        System-controller: set default without matching keys
        '''
        response = self.app.get(url(controller='system', action='setDefault'),
                                params={'wrongKey': 'wrongVal' })
        print "test_set_default:", response
        assert '"status": false' in response
        assert 'Usage: setDefault: parameters are' in response

    def test_setconfig_backwards(self):
        '''
        testing setconfig backward compat
        '''
        response = self.app.get(url(controller='system', action='setConfig'),
                                params={'key': 'test',
                                        'value': 'old',
                                        'description':'old value'})
        print response
        assert '"setConfig test": true' in response

        response = self.app.get(url(controller='system', action='setConfig'),
                                params={'key': 'some.resolver.config',
                                        'value': 'resolverText',
                                        'description':'resolver test'})
        print response
        assert '"setConfig some.resolver.config": true' in response

    def test_0000_setconfig_typing(self):
        '''
        Test: system/setConfig with typing
        '''
        response = self.app.get(url(controller='system', action='setConfig'),
                                params={'secretkey': 'test123',
                                        'secretkey.type': 'password'})
        log.info(response)
        assert '"setConfig secretkey:test123": true' in response

        ## the value will be returned transparently
        response = self.app.get(url(controller='system', action='getConfig'),
                                params={'key': 'secretkey'})
        assert "test123" not in response

        ## the value will be returned transparently
        response = self.app.get(url(controller='system', action='getConfig'),
                                params={'key': 'encprivacyidea.secretkey'})
        assert "test123" in response

        response = self.app.get(url(controller='system', action='delConfig'),
                                params={'key':'secretkey'})
        log.info(response)

        return

    def test_delResolver(self):
        '''
        Testing the deleting of a resolver
        '''
        response = self.app.get(url(controller='system', action='setResolver'),
                                params={'name':'reso1',
                                        'type': 'passwdresolver',
                                        'fileName': '%(here)s/tests/testdata/my-pass2'})
        print response
        assert '"value": true' in response

        response = self.app.get(url(controller='system', action='setResolver'),
                                params={'name':'reso2',
                                        'type': 'passwdresolver',
                                        'fileName': '%(here)s/tests/testdata/my-pass2'})
        print response
        assert '"value": true' in response

        response = self.app.get(url(controller='system', action='setResolver'),
                                params={'name':'reso3',
                                        'type': 'passwdresolver',
                                        'fileName': '%(here)s/tests/testdata/my-pass2'})
        print response
        assert '"value": true' in response

        response = self.app.get(url(controller='system', action='getResolvers'),
                                params={})
        print response
        assert '"entry": "privacyidea.passwdresolver.fileName.reso2"' in response
        assert '"entry": "privacyidea.passwdresolver.fileName.reso1"' in response
        assert '"entry": "privacyidea.passwdresolver.fileName.reso3"' in response

        # create a realm
        response = self.app.get(url(controller='system', action='setRealm'),
                                params={'realm': 'realm1',
                                        'resolvers': 'privacyidea.lib.resolvers.PasswdIdResolver.IdResolver.reso1, privacyidea.lib.resolvers.PasswdIdResolver.IdResolver.reso2'})
        print "create realm: ", response
        assert '"value": true' in response
        
        response = self.app.get(url(controller='system', action='getRealms'),
                                params={})
        print "get realms: ", response
        assert 'privacyidea.useridresolver.group.realm1' in response

        # try to delete a resolver, that is in a realm
        response = self.app.get(url(controller='system', action='delResolver'),
                                params={'resolver': 'reso1'})
        print response
        assert 'Resolver u\'reso1\'  still in use' in response

        response = self.app.get(url(controller='system', action='delResolver'),
                                params={'resolver': 'reso3'})
        print response
        assert '"value": true' in response


    def test_policy_wrong_name(self):
        '''
        testing to set a policy with a wrong name
        '''
        response = self.app.get(url(controller='system', action='setPolicy'),
                                params={'name':'ads ads asd',
                                        'action':'*',
                                        'scope':'admin',
                                        'realm':'*'})
        print response
        assert 'The name of the policy may only contain the characters' in response

    def test_key_value(self):
        response = self.app.get(url(controller='system', action='setConfig') , {"key" : "something"})
        print response
        self.assertTrue("ERR905: Required parameters: value" in response)

        response = self.app.get(url(controller='system', action='setConfig') , {"key" : None})
        print response
        self.assertTrue("ERR905: Required parameters: value" in response)
        
    def test_get_enc_values(self):
        '''
        setConfig: Save encrypted value.
        '''
        response = self.app.get(url(controller='system', action='setConfig') , {"somesecret" : "something",
                                                                                "somesecret.type" : "password"})
        print response
        self.assertTrue('"setConfig somesecret:something": true' in response)
        
        response = self.app.get(url(controller='system', action='getConfig') , {})
        print response
        self.assertTrue("somesecret" in response)

    def test_get_resolver(self):
        response = self.app.get(url(controller='system', action='getResolver') , {"resolver" : "reso1"})
        print response
        self.assertTrue('"resolver": "reso1"' in response)
        
        response = self.app.get(url(controller='system', action='getResolver') , {"resolver" : ""})
        print response
        self.assertTrue('missing resolver name' in response)
        
        response = self.app.get(url(controller='system', action='get_resolver_list') , {})
        print response
        self.assertTrue('privacyidea.lib.resolvers.PasswdIdResolver' in response)
        self.assertTrue('privacyidea.lib.resolvers.SCIMIdResolver' in response)
        self.assertTrue('privacyidea.lib.resolvers.SQLIdResolve' in response)
        self.assertTrue('privacyidea.lib.resolvers.LDAPIdResolver' in response)
        
    def test_set_default_Realm(self):
        response = self.app.get(url(controller='system', action='setDefaultRealm') , {})
        print response
        self.assertTrue('"value": false' in response)
        
    def test_set_realm_error(self):
        response = self.app.get(url(controller='system', action='setRealm') , {"realm" : "errorr",
                                                                               "resolvers" : "asdasd"})
        print response
        self.assertTrue("Error in resolver u'asdasd' please check the logfile!" in response)
        
    def test_empty_policy_name(self):
        response = self.app.get(url(controller='system', action='setPolicy') , {"name" : "",
                                                                                "action" : None,
                                                                                "scope": None,
                                                                                "realm" : None})
        print response
        self.assertTrue("The name of the policy must not be empty" in response)
        
        response = self.app.get(url(controller='system', action='setPolicy') , {"name" : "asdasd",
                                                                                "action" : "",
                                                                                "scope": None,
                                                                                "realm" : None})
        print response
        self.assertTrue("name and action required" in response)
        
    def test_policy_flexi(self):
        response = self.app.get(url(controller='system', action='policies_flexi') , {})
        print response
        self.assertTrue('"status": false' in response)
        
        response = self.app.get(url(controller='system', action='policies_flexi') , {"sortorder" : "desc",
                                                                                     "sortname" : "name"})
        print response
        self.assertTrue('webprovisionGOOGLE,' in response)
        
    def test_getPolicyDef(self):
        response = self.app.get(url(controller='system', action='getPolicyDef') , {})
        print response
        self.assertTrue('totp_hashlib' in response)
        
    def test_checkPolicy(self):
        response = self.app.get(url(controller='system', action='checkPolicy') , {"user" : "*",
                                                                                  "realm" : "myDefRealm",
                                                                                  "scope" : "selfservice",
                                                                                  "action" : "disable",
                                                                                  "client" : "172.0.0.1"})
        print response
        self.assertTrue('"allowed": false' in response)
        
        response = self.app.get(url(controller='system', action='checkPolicy') , {"user" : "superadmin",
                                                                                  "realm" : "myDefRealm",
                                                                                  "scope" : "admin",
                                                                                  "action" : "disable",
                                                                                  "client" : "172.0.0.1"})
        print response
        self.assertTrue('No policies in scope admin' in response)
        
        



# -*- coding: utf-8 -*-
#
#  privacyIDEA
#  Aug 12, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  based on the tokenclass.py base class of LinOTP which is
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
This is the token module for the daplug token. It behaves like HOTP,
but uses another OTP format/mapping.
'''
import binascii
from privacyidea.lib.tokens.hmactoken import HmacTokenClass
from privacyidea.lib.log import log_with
from privacyidea.lib.config import getFromConfig

optional = True
required = False

import logging
log = logging.getLogger(__name__)

MAPPING = {"b": "0",
           "c": "1",
           "d": "2",
           "e": "3",
           "f": "4",
           "g": "5",
           "h": "6",
           "i": "7",
           "j": "8",
           "k": "9"}
         
         
def _daplug2digit(daplug_otp):
    hex_otp = ""
    for i in daplug_otp:
        digit = MAPPING.get(i)
        hex_otp += digit
    otp = binascii.unhexlify(hex_otp)
    return otp
         
         
class DaplugTokenClass(HmacTokenClass):
    '''
    daplug token class implementation
    '''

    @classmethod
    def getClassType(cls):
        '''
        getClassType - return the token type shortname

        :return: 'daplug'
        :rtype: string

        '''
        return "daplug"

    @classmethod
    def getClassPrefix(cls):
        return "DPLG"

    @classmethod
    @log_with(log)
    def getClassInfo(cls, key=None, ret='all'):
        '''
        getClassInfo - returns a subtree of the token definition

        :param key: subsection identifier
        :type key: string

        :param ret: default return value, if nothing is found
        :type ret: user defined

        :return: subsection if key exists or user defined
        :rtype: s.o.

        '''
        res = {'type': 'daplug',
               'title': 'Daplug Event Token',
               'description': ("event based OTP token using "
                               "the HOTP algorithm"),
               }

        if key is not None and key in res:
            ret = res.get(key)
        else:
            if ret == 'all':
                ret = res
        return ret

    @log_with(log)
    def __init__(self, a_token):
        '''
        constructor - create a token object

        :param aToken: instance of the orm db object
        :type aToken:  orm object

        '''
        HmacTokenClass.__init__(self, a_token)
        self.setType(u"DAPLUG")
        self.hKeyRequired = True
        return

    @log_with(log)
    def checkOtp(self, anOtpVal, counter, window, options=None):
        '''
        checkOtp - validate the token otp against a given otpvalue

        :param anOtpVal: the otpvalue to be verified
        :type anOtpVal:  string, format: efekeiebekeh

        :param counter: the counter state, that should be verified
        :type counter: int

        :param window: the counter +window, which should be checked
        :type window: int

        :param options: the dict, which could contain token specific info
        :type options: dict

        :return: the counter state or -1
        :rtype: int

        '''
        # convert OTP value
        otp = _daplug2digit(anOtpVal)
        res = HmacTokenClass.checkOtp(self, otp, counter, window, options)
        return res

    @log_with(log)
    def check_otp_exist(self, otp, window=10):
        '''
        checks if the given OTP value is/are values of this very token.
        This is used to autoassign and to determine the serial number of
        a token.

        :param otp: the to be verified otp value
        :type otp: string

        :param window: the lookahead window for the counter
        :type window: int

        :return: counter or -1 if otp does not exist
        :rtype:  int

        '''
        otp = _daplug2digit(otp)
        res = HmacTokenClass.check_otp_exist(self, otp, window)
        return res

    @log_with(log)
    def autosync(self, hmac2Otp, anOtpVal):
        '''
        auto - sync the token based on two otp values
        - internal method to realize the autosync within the
        checkOtp method

        :param hmac2Otp: the hmac object (with reference to the token secret)
        :type hmac2Otp: hmac object

        :param anOtpVal: the actual otp value
        :type anOtpVal: string

        :return: counter or -1 if otp does not exist
        :rtype:  int

        '''
        otp = _daplug2digit(anOtpVal)
        res = HmacTokenClass.autosync(self, hmac2Otp, otp)
        return res

    @log_with(log)
    def resync(self, otp1, otp2, options=None):
        '''
        resync the token based on two otp values
        - external method to do the resync of the token

        :param otp1: the first otp value
        :type otp1: string

        :param otp2: the second otp value
        :type otp2: string

        :param options: optional token specific parameters
        :type options:  dict or None

        :return: counter or -1 if otp does not exist
        :rtype:  int

        '''
        n_otp1 = _daplug2digit(otp1)
        n_otp2 = _daplug2digit(otp2)
        res = HmacTokenClass.resync(self, n_otp1, n_otp2, options)
        return res

    def splitPinPass(self, passw):

        res = 0
        try:
            otplen = int(self.token.privacyIDEAOtpLen)
        except ValueError:
            otplen = 6

        # For splitting the value we use 12 characters.
        # For internal calculation we use 6 digits.
        otplen = otplen * 2
        
        if getFromConfig("PrependPin") == "True":
            pin = passw[0:-otplen]
            otpval = passw[-otplen:]
        else:
            pin = passw[otplen:]
            otpval = passw[0:otplen]

        return (res, pin, otpval)



# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''     
  Description:  functional tests
                
  Dependencies: -

'''

import logging
import random
from datetime import datetime
from datetime import timedelta

import json


from sqlalchemy.engine import create_engine
import sqlalchemy

from privacyidea.tests import TestController, url

log = logging.getLogger(__name__)


class SQLData(object):

    def __init__(self, connect='sqlite:///:memory:'):
        self.userTable = 'Config'

        self.connection = None
        try:
            self.engine = create_engine(connect)
            connection = self.engine.connect()
        except Exception as e:
            print "%r" % e
        self.connection = connection
        return

    def addData(self, key, value, typ, description):
        iStr = """
            INSERT INTO "%s"( "Key", "Value", "Type", "Description")
            VALUES (:key, :value, :typ, :description);
            """ % (self.userTable)

        if "mysql" in self.engine.driver:
            iStr = """
            INSERT INTO %s (%s.Key, Value, Type, Description)
            VALUES (:key, :value, :typ, :description);
            """ % (self.userTable, self.userTable)


        intoStr = iStr

        t = sqlalchemy.sql.expression.text(intoStr)

        self.connection.execute(t, key=key, value=value, typ=typ, description=description)
        return

    def updateData(self, key, value):
        uStr = 'UPDATE "%s"  SET "Value"=:value WHERE "Key" = :key;'
        if "mysql" in self.engine.driver:
            uStr = 'UPDATE %s  SET Value=:value WHERE Config.Key = :key;'

        updateStr = uStr % (self.userTable)

        t = sqlalchemy.sql.expression.text(updateStr)
        self.connection.execute(t, key=key, value=value)
        return

    def query(self):
        selectStr = "select * from %s" % (self.userTable)
        result = self.connection.execute(selectStr)
        rows = []
        for row in result:
            rows.append(row)
            print unicode(row)
        return

    def delData(self, key):
        dStr = 'DELETE FROM "%s" WHERE "Key"=:key;' % (self.userTable)
        if "mysql" in self.engine.driver:
            dStr = ('DELETE FROM %s WHERE %s.Key=:key;' %
                    (self.userTable, self.userTable))

        delStr = dStr
        t = sqlalchemy.sql.expression.text(delStr)
        self.connection.execute(t, key=key)
        return


    def close(self):
        self.connection.close()


    def __del__(self):
        self.connection.close()






class TestReplication(TestController):

    def setUp(self):

        #self.appconf = self.app.app.app.apps[1].application.app.application.app.app.app.config
        self.sqlconnect = self.appconf.get('sqlalchemy.url')
        sqlData = SQLData(connect=self.sqlconnect)
        log.debug(sqlData)

        return

    def tearDown(self):
        ''' Overwrite parent tear down, which removes all realms '''
        return

    def addData(self, key, value, description):

        sqlData = SQLData(connect=self.sqlconnect)
        typ = type(value).__name__
        sqlData.addData(key, value, typ, description)
        sec = random.randrange(1, 9)
        sqlData.updateData("privacyidea.Config", str(datetime.now()
                                               + timedelta(milliseconds=sec)))
        sqlData.close()

        return


    def delData(self, key):

        sqlData = SQLData(connect=self.sqlconnect)
        sqlData.delData(key)

        sec = random.randrange(1, 9)
        sqlData.updateData("privacyidea.Config", str(datetime.now()
                                               + timedelta(milliseconds=sec)))
        sqlData.close()

        return

    def addToken(self, user):

        param = { 'user': user, 'pin':user, 'serial': 's' + user, 'type':'spass' }
        response = self.app.get(url(controller='admin', action='init'), params=param)
        assert '"status": true,' in response

        return

    def authToken(self, user):

        param = { 'user': user, 'pass':user}
        response = self.app.get(url(controller='validate', action='check'), params=param)
        return response

    def showTokens(self):

        param = None
        response = self.app.get(url(controller='admin', action='show'), params=param)
        assert '"status": true,' in response
        return response


    def test_replication(self):
        '''
            test replication of an simple config entry

            Description:
            - put privacyIDEA in replication aware mode
            - add a new entry in the Config Data vi SQL + update the timestamp
            - query the Config (system/getConfig, which should show the entry

            - del a entry in the Config Data vi SQL + update the timestamp
            - query the Config (system/getConfig, which should show the entry no more

        '''
        ''' 0. '''
        parameters = {'enableReplication' : 'true' }
        resp = self.app.get(url(controller='system', action='setConfig'), params=parameters)
        assert('"setConfig enableReplication:true": true' in resp)

        ''' 1. '''
        self.addData('replication', 'test1', 'test data')

        ''' 2. '''
        resp = self.app.get(url(controller='system', action='getConfig'))
        assert('"replication": "test1"' in resp)


        ''' 3. '''
        self.delData('replication')

        ''' 4. '''
        resp = self.app.get(url(controller='system', action='getConfig'))
        res = ('"replication": "test1"' in resp)
        assert (res == False)

        ''' 5 - cleanup'''
        parameters = {'key':'enableReplication' }
        resp = self.app.get(url(controller='system', action='delConfig'), params=parameters)
        assert('"delConfig enableReplication": true' in resp)

        return



    def test_replication_2(self):
        '''
            test 'no' replication, when 'enableReplication' entry is not set

            Description:
            - put privacyIDEA in replication aware mode
            - add a new entry in the Config Data vi SQL + update the timestamp
            - query the Config (system/getConfig, which should show the entry

            - del a entry in the Config Data vi SQL + update the timestamp
            - query the Config (system/getConfig, which should show the entry no more

        '''
        ''' 0. '''
        self.addData('replication', 'test1', 'test data')

        ''' 1. '''
        resp = self.app.get(url(controller='system', action='getConfig'))
        res = ('"replication": "test1"' in resp)
        assert (res == False)

        ''' 2. '''
        parameters = {'enableReplication' : 'true' }
        resp = self.app.get(url(controller='system', action='setConfig'), params=parameters)
        assert('"setConfig enableReplication:true": true' in resp)


        ''' 3. '''
        self.delData('replication')

        ''' 3. '''
        resp = self.app.get(url(controller='system', action='getConfig'))
        res = ('"replication": "test1"' in resp)
        assert (res == False)


        self.addData('replication', 'test1', 'test data')

        ''' 4. '''
        resp = self.app.get(url(controller='system', action='getConfig'))
        res = ('"replication": "test1"' in resp)
        assert (res == True)


        ''' 3. '''
        self.delData('replication')

        ''' 3. '''
        resp = self.app.get(url(controller='system', action='getConfig'))
        res = ('"replication": "test1"' in resp)
        assert (res == False)


        ''' 5 - cleanup'''
        parameters = {'key':'enableReplication' }
        resp = self.app.get(url(controller='system', action='delConfig'), params=parameters)
        assert('"delConfig enableReplication": true' in resp)

        return


    def test_updateResolver(self):
        '''
            test replication with resolver update

        '''
        #FIXME missing SQLresolver
        return
        umap = { "userid" : "id",
                "username": "user",
                "phone" : "telephoneNumber",
                "mobile" : "mobile",
                "email" : "mail",
                "surname" : "sn",
                "givenname" : "givenName",
                "password" : "password",
                "salt" : "salt" }

        sqlResolver = {
            "sqlresolver.conParams.mySQL": None,
            "sqlresolver.Where.mySQL": None,
            "sqlresolver.Limit.mySQL": "20",
            "sqlresolver.User.mySQL": "user",
            "sqlresolver.Database.mySQL": "yourUserDB",
            "sqlresolver.Password.mySQL": "157455c27f605ad309d6059e1d936a4" +
                                        "e:7a812ba9e613fb931386f5f4fb025890",
            "sqlresolver.Table.mySQL": "usertable",
            "sqlresolver.Server.mySQL": "127.0.0.1",
            "sqlresolver.Driver.mySQL": "mysql",
            "sqlresolver.Encoding.mySQL": None,
            "sqlresolver.Port.mySQL": "3306",
            "sqlresolver.Map.mySQL": json.dumps(umap)

            }
        for k in sqlResolver:
            self.delData(k)

        ''' 0. '''
        parameters = {'enableReplication' : 'true' }
        resp = self.app.get(url(controller='system', action='setConfig'),
                            params=parameters)
        assert('"setConfig enableReplication:true": true' in resp)

        for k in sqlResolver:
            self.addData(k, sqlResolver.get(k), '')

        param = {'resolver':'mySQL'}
        resp = self.app.get(url(controller='system', action='getResolver'),
                            params=param)
        assert('"Database": "yourUserDB"' in resp)

        for k in sqlResolver:
            self.delData(k)

        param = {'resolver':'mySQL'}
        resp = self.app.get(url(controller='system', action='getResolver'),
                            params=param)
        assert('"data": {}' in resp)


        ''' 5 - cleanup'''
        parameters = {'key':'enableReplication' }
        resp = self.app.get(url(controller='system', action='delConfig'),
                            params=parameters)
        assert('"delConfig enableReplication": true' in resp)



        return

    def test_updateRealm(self):
        '''
            test replication with realm and resolver update
        '''
        realmDef = {
            "useridresolver.group.realm":
                    "privacyidea.lib.resolvers.PasswdIdResolver.IdResolver.resolverTest",
            "passwdresolver.fileName.resolverTest": "/etc/passwd",
            "DefaultRealm": "realm",
            }

        for k in realmDef:
            self.delData(k)

        parameters = {'enableReplication' : 'true' }
        resp = self.app.get(url(controller='system', action='setConfig'),
                            params=parameters)
        assert('"setConfig enableReplication:true": true' in resp)


        resp = self.app.get(url(controller='system', action='getRealms'))
        res = '"realmname": "realm"' in resp
        assert res == False


        for k in realmDef:
            self.addData(k, realmDef.get(k), '')

        resp = self.app.get(url(controller='system', action='getRealms'))
        res = '"realmname": "realm"' in resp
        assert res == True


        ''' 5 - cleanup'''
        for k in realmDef:
            self.delData(k)

        resp = self.app.get(url(controller='system', action='getRealms'))
        res = '"realmname": "realm"' in resp
        assert res == False


        parameters = {'key':'enableReplication' }
        resp = self.app.get(url(controller='system', action='delConfig'),
                            params=parameters)
        assert('"delConfig enableReplication": true' in resp)



        return

    def test_auth_updateRealm(self):
        '''
          test resolver and realm update with authentication

          0. delete all related data
          1. enable replication
          2. write sql data
          3. lookup for the realm definition
          4. enroll token and auth for user root
          5. cleanup: remove realm definition + replication flag

        '''
        realmDef = {
            "useridresolver.group.realm":
                    "privacyidea.lib.resolvers.PasswdIdResolver.IdResolver.resolverTest",
            "passwdresolver.fileName.resolverTest": "/etc/passwd",
            "DefaultRealm": "realm",
            }

        ''' 0. delete all related data'''
        for k in realmDef:
            self.delData(k)

        ''' 1. switch on replication '''
        parameters = {'enableReplication' : 'true' }
        resp = self.app.get(url(controller='system', action='setConfig'),
                            params=parameters)
        assert('"setConfig enableReplication:true": true' in resp)


        ''' 1.b check that realm is not defined '''
        resp = self.app.get(url(controller='system', action='getRealms'))
        res = '"realmname": "realm"' in resp
        assert res == False


        ''' 2  write sql data '''
        for k in realmDef:
            self.addData(k, realmDef.get(k), '')

        ''' 3. lookup for the realm definition'''
        resp = self.app.get(url(controller='system', action='getRealms'))
        res = '"realmname": "realm"' in resp
        assert res == True

        ''' 4. enroll token and auth for user root '''
        self.addToken('root')
        res = self.authToken('root')
        assert ('"value": true' in res)


        ''' 5 - cleanup'''
        for k in realmDef:
            self.delData(k)

        ''' 5b. lookup for the realm definition'''
        resp = self.app.get(url(controller='system', action='getRealms'))
        res = '"realmname": "realm"' in resp
        assert res == False

        parameters = {'key':'enableReplication' }
        resp = self.app.get(url(controller='system', action='delConfig'),
                            params=parameters)
        assert('"delConfig enableReplication": true' in resp)


        return


    def test_0000_policy(self):
        '''
            test the replication of policies
        '''

        policyDef = {
            "Policy.enrollPolicy.action": "maxtoken=3,",
            "Policy.enrollPolicy.scope": "enrollment",
            "Policy.enrollPolicy.client": None,
            "Policy.enrollPolicy.time": None,
            "Policy.enrollPolicy.realm": "*",
            "Policy.enrollPolicy.user": "*",
            }

        ''' 0 - cleanup'''
        parameters = {'key':'enableReplication' }
        resp = self.app.get(url(controller='system', action='delConfig'),
                            params=parameters)
        assert('"delConfig enableReplication": true' in resp)

        for k in policyDef:
            self.delData(k)


        ''' 1. switch on replication '''
        parameters = {'enableReplication' : 'true' }
        resp = self.app.get(url(controller='system', action='setConfig'),
                            params=parameters)
        assert('"setConfig enableReplication:true": true' in resp)

        ''' 2  write sql data '''
        for k in policyDef:
            self.addData(k, policyDef.get(k), '')

        ''' 3. getPolicy '''
        parameters = {'name' : 'enrollPolicy' }
        resp = self.app.get(url(controller='system', action='getPolicy'),
                            params=parameters)
        assert('"action": "maxtoken=3' in resp)

        ''' 5 - cleanup'''
        for k in policyDef:
            self.delData(k)

        ''' 5b. lookup for the policy definition'''
        parameters = {'name' : 'enrollPolicy' }
        resp = self.app.get(url(controller='system', action='getPolicy'),
                            params=parameters)
        res = ('"action": "maxtoken=3' in resp)
        assert res == False

        ''' 5c. reset replication '''
        parameters = {'key':'enableReplication' }
        resp = self.app.get(url(controller='system', action='delConfig'),
                            params=parameters)
        assert('"delConfig enableReplication": true' in resp)


        return

##eof##########################################################################




# -*- coding:utf-8 -*-
from mako import runtime, filters, cache
UNDEFINED = runtime.UNDEFINED
__M_dict_builtin = dict
__M_locals_builtin = locals
_magic_number = 9
_modified_time = 1422216484.925574
_enable_loop = True
_template_filename = '/usr/lib/python2.7/dist-packages/privacyidea/lib/tokens/tagespassworttoken.mako'
_template_uri = '/tagespassworttoken.mako'
_source_encoding = 'utf-8'
from webhelpers.html import escape
_exports = []


def render_body(context,**pageargs):
    __M_caller = context.caller_stack._push_frame()
    try:
        __M_locals = __M_dict_builtin(pageargs=pageargs)
        c = context.get('c', UNDEFINED)
        _ = context.get('_', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 2
        __M_writer(u'\n')
        # SOURCE LINE 3
        if c.scope == 'enroll.title' :
            # SOURCE LINE 4
            __M_writer(escape(_("Day OTP Token / Tagespasswort")))
            __M_writer(u'\n')
        # SOURCE LINE 6
        __M_writer(u'\n')
        # SOURCE LINE 7
        if c.scope == 'enroll' :
            # SOURCE LINE 8
            __M_writer(u"\n<script>\n/*\n * 'typ'_get_enroll_params()\n *\n * this method is called, when the token  is submitted\n * - it will return a hash of parameters for admin/init call\n *\n */\n\nfunction dpw_get_enroll_params(){\n    var params = {};\n    params['type'] = 'dpw';\n    //params['serial'] = create_serial('DOTP');\n    params['otpkey'] \t= $('#dpw_key').val();\n\tparams['description'] =  $('#enroll_dpw_desc').val();\n\n\tjQuery.extend(params, add_user_data());\n\n    return params;\n}\n</script>\n\n<p>")
            # SOURCE LINE 31
            __M_writer(escape(_("Here you can define the 'Tagespasswort' token, that changes every day.")))
            __M_writer(u'</p>\n<table>\n<tr>\n\t<td><label for="dpw_key">')
            # SOURCE LINE 34
            __M_writer(escape(_("DPW key")))
            __M_writer(u'</label></td>\n\t<td><input type="text" name="dpw_key" id="dpw_key" value="" class="text ui-widget-content ui-corner-all" /></td>\n</tr>\n<tr>\n    <td><label for="enroll_dpw_desc" id=\'enroll_dpw_desc_label\'>')
            # SOURCE LINE 38
            __M_writer(escape(_("Description")))
            __M_writer(u'</label></td>\n    <td><input type="text" name="enroll_dpw_desc" id="enroll_dpw_desc" value="webGUI_generated" class="text" /></td>\n</tr>\n</table>\n\n\n')
        return ''
    finally:
        context.caller_stack._pop_frame()





# -*- coding: utf-8 -*-
#VERSION = "1.5dev9"
VERSION = "1.5.1.1"
try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages

import os
import sys

# Taken from kennethreitz/requests/setup.py
package_directory = os.path.realpath(os.path.dirname(__file__))


def get_file_contents(file_path):
    """Get the context of the file using full path name."""
    content = ""
    try:
        full_path = os.path.join(package_directory, file_path)
        content = open(full_path, 'r').read()
    except:
        print >> sys.stderr, "### could not open file: %r" % file_path
    return content


def get_package():
    """
    returns a tuple of path.
    returns ("/", "/usr") if we do a pacakge installation.
    Then a file PRIVACYIDEA_PACKAGE needs to be created.
    """
    check_file = os.path.join(package_directory, "PRIVACYIDEA_PACKAGE")
    if os.path.isfile(check_file):
        return ("/", "/usr/")
    return ("", "")


setup(
    name='privacyIDEA',
    version=VERSION,
    description='privacyIDEA: identity, multifactor authentication, '
                'authorization, audit',
    author='privacyidea.org',
    license='AGPL v3',
    author_email='cornelius@privacyidea.org',
    url='http://www.privacyidea.org',
    install_requires=["WebOb>=1.2,<=1.3",
                      "Pylons>=0.9.7,<=1.0",
                      "SQLAlchemy>=0.6",
                      "docutils>=0.4",
                      "simplejson>=2.0",
                      "pycrypto>=1.0",
                      "repoze.who<=1.1",
                      "pyrad>=1.1",
                      "netaddr",
                      "qrcode>=2.4",
                      "configobj>=4.6.0",
                      "httplib2",
                      "pyyaml",
                      "python-ldap",
                      "Pillow",
                      "sqlsoup",
                      "pyparsing"
                      ],
    scripts=['tools/privacyidea-convert-token',
             'tools/privacyidea-create-pwidresolver-user',
             'tools/privacyidea-create-sqlidresolver-user',
             'tools/privacyidea-pip-update',
             'tools/privacyidea-create-enckey',
             'tools/privacyidea-create-auditkeys',
             'tools/privacyidea-create-certificate',
             'tools/privacyidea-create-database',
             'tools/privacyidea-fix-access-rights',
             'tools/totp-token',
             'tools/privacyidea-create-ad-users',
             'tools/privacyidea-setup',
             'tools/privacyidea-backup',
             'tools/privacyidea-restore'
             ],
    setup_requires=["PasteScript>=1.6.3"],
    packages=find_packages(exclude=['ez_setup']),
    include_package_data=True,
    test_suite='nose.collector',
    data_files=[(get_package()[0] + 'etc/privacyidea/',
                 ['config/privacyidea.ini.example',
                  'config/privacyideaapp.wsgi',
                  'config/privacyideaapp.py',
                  'config/dictionary',
                  'config/dummy-encKey']),
                (get_package()[1] + 'share/man/man1',
                 ["tools/privacyidea-convert-token.1",
                  "tools/privacyidea-create-pwidresolver-user.1",
                  "tools/privacyidea-create-sqlidresolver-user.1",
                  "tools/totp-token.1",
                  "tools/privacyidea-pip-update.1",
                  "tools/privacyidea-create-enckey.1",
                  "tools/privacyidea-create-auditkeys.1",
                  "tools/privacyidea-create-certificate.1",
                  "tools/privacyidea-create-database.1",
                  "tools/privacyidea-fix-access-rights.1"
                  ]),
                (get_package()[1] + 'lib/privacyidea/authmodules/FreeRADIUS',
                 ["authmodules/FreeRADIUS/LICENSE",
                  "authmodules/FreeRADIUS/privacyidea_radius.pm"]),
                (get_package()[1] + 'lib/privacyidea/authmodules/OTRS',
                 ["authmodules/OTRS/privacyIDEA.pm"])
                ],
    classifiers=["Framework :: Pylons",
                 "License :: OSI Approved :: "
                 "GNU Affero General Public License v3",
                 "Programming Language :: Python",
                 "Development Status :: 5 - Production/Stable",
                 "Topic :: Internet",
                 "Topic :: Security",
                 "Topic :: System ::"
                 " Systems Administration :: Authentication/Directory"
                 ],
    message_extractors={'privacyidea': [
            ('**.py', 'python', None),
            ('templates/**.mako', 'mako', {'input_encoding': 'utf-8'}),
            ('lib/tokens/*.mako', 'mako', {'input_encoding': 'utf-8'}),
            ('public/**', 'ignore', None)]},
    zip_safe=False,
    paster_plugins=['PasteScript', 'Pylons'],
    entry_points="""
    [paste.app_factory]
    main = privacyidea.config.middleware:make_app

    [paste.app_install]
    main = pylons.util:PylonsInstaller

    [nose.plugins]
    pylons = pylons.test:PylonsPlugin
    """,
    long_description=get_file_contents('README.md')
)
